Package: autotestR
Title: Automated Functions for Basic Statistical Tests
Version: 1.2.15
Authors@R: 
    person("Luiz", "Garcia", 
           email = "luiz.cardoso@ufpr.br", 
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-9616-0927"))
Description: Provides simple and intuitive functions for basic statistical analyses. 
    Methods include the t-test (Student 1908 <doi:10.1093/biomet/6.1.1>),
    the Mann-Whitney U test (Mann and Whitney 1947 <doi:10.1214/aoms/1177730491>),
    Pearson's correlation (Pearson 1895 <doi:10.1098/rspl.1895.0041>),
    and analysis of variance (Fisher 1925, <doi:10.1007/978-1-4612-4380-9_5>).
    Functions are compatible with 'ggplot2' and 'dplyr'.
License: MIT + file LICENSE
Depends: R (>= 4.1.0)
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: car, crayon, dplyr, FSA, ggplot2, ggdist, ggExtra, nortest,
        purrr, RColorBrewer, scales, stats, tibble, tidyr, tidyselect,
        utils, vcd
NeedsCompilation: no
Maintainer: Luiz Garcia <luiz.cardoso@ufpr.br>
Packaged: 2026-04-24 01:37:19 UTC; Seu Computador
Author: Luiz Garcia [aut, cre] (ORCID: <https://orcid.org/0000-0002-9616-0927>)
Repository: CRAN
Date/Publication: 2026-04-28 18:30:16 UTC
Built: R 4.5.2; ; 2026-05-01 19:37:19 UTC; unix

Package: autotestR
Title: Automated Functions for Basic Statistical Tests
Version: 1.2.16
Authors@R: 
    person("Luiz", "Garcia", 
           email = "luiz.cardoso@ufpr.br", 
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-9616-0927"))
Description: Provides simple and intuitive functions for basic statistical analyses. 
    Methods include the t-test (Student 1908 <doi:10.1093/biomet/6.1.1>),
    the Mann-Whitney U test (Mann and Whitney 1947 <doi:10.1214/aoms/1177730491>),
    Pearson's correlation (Pearson 1895 <doi:10.1098/rspl.1895.0041>),
    and analysis of variance (Fisher 1925, <doi:10.1007/978-1-4612-4380-9_5>).
    Functions are compatible with 'ggplot2' and 'dplyr'.
License: MIT + file LICENSE
Depends: R (>= 4.1.0)
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: car, crayon, dplyr, FSA, ggplot2, ggdist, ggExtra, nortest,
        purrr, RColorBrewer, scales, stats, tibble, tidyr, tidyselect,
        utils, vcd
NeedsCompilation: no
Maintainer: Luiz Garcia <luiz.cardoso@ufpr.br>
Packaged: 2026-08-20 12:13:52 UTC; Seu Computador
Author: Luiz Garcia [aut, cre] (ORCID: <https://orcid.org/0000-0002-9616-0927>)
Repository: CRAN
Date/Publication: 2026-08-21 05:45:17 UTC
Built: R 4.6.1; ; 2026-08-21 09:45:12 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,test.correlation)
S3method(print,test.interaction)
export(pre.test)
export(test.anova)
export(test.chi)
export(test.correlation)
export(test.fisher)
export(test.interaction)
export(test.kruskal)
export(test.t)
export(test.tmulti)
export(test.tpaired)
export(test.u)
importFrom(RColorBrewer,brewer.pal)
importFrom(car,leveneTest)
importFrom(dplyr,"%>%")
importFrom(ggExtra,ggMarginal)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,theme_minimal)
importFrom(grDevices,gray.colors)
importFrom(purrr,map_dfr)
importFrom(scales,hue_pal)
importFrom(stats,TukeyHSD)
importFrom(stats,aggregate)
importFrom(stats,aov)
importFrom(stats,chisq.test)
importFrom(stats,complete.cases)
importFrom(stats,confint)
importFrom(stats,dist)
importFrom(stats,formula)
importFrom(stats,kruskal.test)
importFrom(stats,ks.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,nobs)
importFrom(stats,p.adjust)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,reformulate)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,shapiro.test)
importFrom(stats,t.test)
importFrom(stats,var.test)
importFrom(stats,wilcox.test)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,pivot_longer)
importFrom(tidyselect,everything)
importFrom(utils,combn)
importFrom(utils,globalVariables)

# autotestR

<!-- badges: start -->
<!-- badges: end -->

## Overview

**autotestR** is an R package designed to simplify and improve statistical analysis
in the life sciences.

It combines automatic test selection, diagnostic evaluation,
effect size reporting, and intuitive visualization to support
transparent and responsible data interpretation.


## Installation

You can install the development version of autotestR directly from GitHub:

```r
# Install autotestR from GitHub

install.packages("remotes")
remotes::install_github("Luiz-Garcia-R/autotestR")

# Install autotestR from CRAN
install.packages("autotestR")
```


## Philosophy

autotestR is designed to go beyond "p-value driven" analysis.

The package emphasizes:

- Effect sizes and confidence intervals
- Automatic diagnostic checks
- Warnings when test assumptions are violated
- Transparent reporting of uncertainty
- Encouragement of biological interpretation over mechanical decision-making

The goal is to support reproducible, interpretable, and responsible data analysis in the life sciences.


## Main features

- Automatic selection and execution of classical statistical tests
- Integrated assumption diagnostics and warnings
- Effect size estimation with confidence intervals
- Robust alternatives for non-normal data
- Built-in graphical summaries
- Consistent and interpretable outputs
- Support for categorical, continuous, and correlation analyses


## What makes autotestR different?

Unlike many statistical wrappers that focus mainly on hypothesis testing,
autotestR prioritizes interpretation.

Instead of providing only p-values, the package:

- Reports effect sizes whenever possible
- Highlights uncertainty
- Flags potential violations of assumptions
- Encourages users to look beyond statistical significance

This makes autotestR especially suitable for exploratory and applied research in biology, medicine, and veterinary sciences.


## Automatic diagnostics and warnings

Many functions in autotestR automatically evaluate key assumptions
(e.g., normality, homoscedasticity, expected frequencies).

When potential issues are detected, the user is informed through
clear warnings and messages, helping prevent inappropriate test usage.

### Basic usage

```r
library(autotestR)

# Independent t test
group1 <- rnorm(30, 10, 2)
group2 <- rnorm(30, 12, 2)
test.t(group1, group2)

# Chi-squared test
var1 <- sample(c("A", "B"), 100, replace = TRUE)
var2 <- sample(c("Yes", "No"), 100, replace = TRUE)
test.chi(var1, var2)

# Multiple test (t test or Mann–Whitney)
df <- data.frame(
  control   = rnorm(30, 10),
  treatment = rnorm(30, 12),
  test1     = rnorm(30, 11),
  test2     = rnorm(30, 15)
)
test.tmulti(df)

# ANOVA with post hoc test
g1 <- rnorm(20, 5)
g2 <- rnorm(20, 7)
g3 <- rnorm(20, 6)
test.anova(g1, g2, g3)

# Correlation test
x <- rnorm(30)
y <- x + rnorm(30, 0, 1)
test.correlation(x, y)
```


# Contact

If you have questions, suggestions, or would like to contribute, feel free to open an issue or submit a pull request on the GitHub repository.

Thank you for using autotestR!

# autotestR

<!-- badges: start -->
<!-- badges: end -->

## Overview

**autotestR** is an R package designed to simplify and improve statistical analysis
in the life sciences.

It combines automatic test selection, diagnostic evaluation,
effect size reporting, and intuitive visualization to support
transparent and responsible data interpretation.


## Installation

You can install the development version of autotestR directly from GitHub:

```r
# Install remotes if you don't have it yet
install.packages("remotes")

# Install autotestR from GitHub
remotes::install_github("Luiz-Garcia-R/autotestR")
```


## Philosophy

autotestR is designed to go beyond "p-value driven" analysis.

The package emphasizes:

- Effect sizes and confidence intervals
- Automatic diagnostic checks
- Warnings when test assumptions are violated
- Transparent reporting of uncertainty
- Encouragement of biological interpretation over mechanical decision-making

The goal is to support reproducible, interpretable, and responsible data analysis in the life sciences.


## Main features

- Automatic selection and execution of classical statistical tests
- Integrated assumption diagnostics and warnings
- Effect size estimation with confidence intervals
- Robust alternatives for non-normal data
- Built-in graphical summaries
- Consistent and interpretable outputs
- Support for categorical, continuous, and correlation analyses


## What makes autotestR different?

Unlike many statistical wrappers that focus mainly on hypothesis testing,
autotestR prioritizes interpretation.

Instead of providing only p-values, the package:

- Reports effect sizes whenever possible
- Highlights uncertainty
- Flags potential violations of assumptions
- Encourages users to look beyond statistical significance

This makes autotestR especially suitable for exploratory and applied research in biology, medicine, and veterinary sciences.


## Automatic diagnostics and warnings

Many functions in autotestR automatically evaluate key assumptions
(e.g., normality, homoscedasticity, expected frequencies).

When potential issues are detected, the user is informed through
clear warnings and messages, helping prevent inappropriate test usage.

### Basic usage

```r
library(autotestR)

# Independent t test
group1 <- rnorm(30, 10, 2)
group2 <- rnorm(30, 12, 2)
test.t(group1, group2)

# Chi-squared test
var1 <- sample(c("A", "B"), 100, replace = TRUE)
var2 <- sample(c("Yes", "No"), 100, replace = TRUE)
test.chi(var1, var2)

# Multiple test (t test or Mann–Whitney)
df <- data.frame(
  control   = rnorm(30, 10),
  treatment = rnorm(30, 12),
  test1     = rnorm(30, 11),
  test2     = rnorm(30, 15)
)
test.tmulti(df)

# ANOVA with post hoc test
g1 <- rnorm(20, 5)
g2 <- rnorm(20, 7)
g3 <- rnorm(20, 6)
test.anova(g1, g2, g3)

# Correlation test
x <- rnorm(30)
y <- x + rnorm(30, 0, 1)
test.correlation(x, y)
```


# Contact

If you have questions, suggestions, or would like to contribute, feel free to open an issue or submit a pull request on the GitHub repository.

Thank you for using autotestR!

#' Mann-Whitney U Test
#'
#' Performs the Mann-Whitney (Wilcoxon rank-sum) test for comparing two independent groups,
#' with statistical summary and graphical visualization.
#'
#' @param ... Two numeric vectors or a data.frame with two numeric columns.
#' @param title Plot title. Default: "Mann-Whitney Test".
#' @param xlab Label for x-axis. Default: "Group".
#' @param ylab Label for y-axis. Default: "Value".
#' @param style Plot aesthetic style.
#' @param help Logical. If TRUE, prints a detailed explanation. Default: FALSE.
#' @param verbose Logical. If TRUE, prints detailed messages. Default: TRUE.
#' @importFrom stats median
#'
#' @return Invisible list with:
#' \describe{
#'   \item{summary}{Group-wise statistical summary}
#'   \item{test}{Test result (htest object)}
#'   \item{plot}{ggplot2 visualization object}
#' }
#' @export
#'
#' @examples
#' x <- c(1, 3, 5, 6)
#' y <- c(7, 8, 9, 12)
#' data <- data.frame(groupA = x, groupB = y)
#' test.u(data)

test.u <- function(...,
                   title = "Mann-Whitney Test",
                   xlab = "Group",
                   ylab = "Value",
                   style = c("boxplot", "violin", "mono", "halfeye"),
                   help = FALSE,
                   verbose = TRUE) {

  input_groups <- list(...)
  style <- match.arg(style)

  # ============================
  # Input via data.frame
  # ============================
  if (length(input_groups) == 1 && is.data.frame(input_groups[[1]])) {

    df <- input_groups[[1]]

    if (ncol(df) != 2)
      stop("The data.frame must contain exactly two numeric columns.")

    if (!all(vapply(df, is.numeric, logical(1))))
      stop("Both columns must be numeric.")

    group_names <- colnames(df)
    groups <- as.list(df)

  } else {

    if (length(input_groups) != 2)
      stop("Provide two numeric vectors or one data.frame with two columns.")

    if (!all(vapply(input_groups, is.numeric, logical(1))))
      stop("All groups must be numeric vectors.")

    call_names <- as.character(match.call(expand.dots = FALSE)$...)
    group_names <- sub("^.*\\$", "", call_names)
    groups <- input_groups
  }

  # ============================
  # Help message
  # ============================
  if (help) {

    if (verbose) {
      message("
Function: test.u()

Description:
  Performs the Mann-Whitney (Wilcoxon rank-sum) test to compare two independent groups.

When to use:
  - Non-normal or ordinal data
  - Comparison of two independent groups
  - Non-parametric alternative to the t-test

Examples:
  x <- c(1, 3, 5, 6)
  y <- c(7, 8, 9, 12)
  data <- data.frame(groupA = x, groupB = y)
  test.u(data)
")
    }

    return(invisible(NULL))
  }

  # ============================
  # Package checking
  # ============================
  required_packages <- c("ggplot2", "dplyr", "scales", "ggdist")

  for (pkg in required_packages) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
      stop(
        paste0(
          "Package ", pkg,
          " is not installed. Install it with install.packages('", pkg, "')"
        )
      )
    }
  }

  # ============================
  # Long-format data
  # ============================
  values <- unlist(groups)

  group_factor <- factor(
    rep(group_names, times = vapply(groups, length, integer(1))),
    levels = group_names
  )

  data_long <- data.frame(
    value = values,
    group = group_factor
  )

  # ============================
  # Mann-Whitney test
  # ============================
  test_result <- stats::wilcox.test(
    groups[[1]],
    groups[[2]],
    exact = FALSE
  )

  p_value <- test_result$p.value
  p_label <- .format_pval(p_value)

  x_data <- groups[[1]]
  y_data <- groups[[2]]

  nx <- sum(!is.na(x_data))
  ny <- sum(!is.na(y_data))

  # ============================
  # Median difference
  # ============================
  median_diff <- median(x_data, na.rm = TRUE) - median(y_data, na.rm = TRUE)

  # ============================
  # Rank-biserial correlation
  # ============================
  U <- test_result$statistic

  r_rb <- as.numeric((2 * U) / (nx * ny) - 1)

  # ============================
  # Bootstrap CI (median diff)
  # ============================

  res_boot <- .boot_two_sample(
    x_data,
    y_data,
    stat_fun = function(a, b)
      median(a, na.rm = TRUE) - median(b, na.rm = TRUE)
  )

  ci_low  <- res_boot$ci_low
  ci_high <- res_boot$ci_high

  # ============================
  # Labels and colors
  # ============================
  p_label <- signif(p_value, 3)

  signif_label <- if (p_value < 0.001) {
    "***"
  } else if (p_value < 0.01) {
    "**"
  } else if (p_value < 0.05) {
    "*"
  } else {
    ""
  }

  y_pos <- max(values, na.rm = TRUE) +
    0.1 * diff(range(values, na.rm = TRUE))

  vivid_colors <- scales::hue_pal()(length(unique(data_long$group)))
  mono_colors <- c("grey75", "grey25")

  # ============================
  # STYLE 1 — Boxplot + jitter
  # ============================
  if (style == "boxplot") {
    g <- ggplot2::ggplot(data_long, ggplot2::aes(x = group, y = value, fill = group)) +
      ggplot2::geom_boxplot(alpha = 0.75, outlier.shape = NA, width = 0.60) +
      ggplot2::geom_jitter(width = 0.1, alpha = 0.2, color = "grey25", size = 1.8) +
      ggplot2::annotate(
        "text", x = mean(1:2), y = y_pos,
        label = signif_label, size = 6,
        col = "grey25"
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::labs(
        title = title,
        subtitle = .build_subtitle_u(median_diff, p_value),
        x = "",
        y = ylab
      ) +
      ggplot2::theme(
        legend.position = "none",
        plot.margin = ggplot2::margin(5.5, 5.5, 10, 5.5),
        axis.text.x = ggplot2::element_text(
          angle = 45, hjust = 1, size = 12
        )
      )
  }

  # ============================
  # STYLE 2 — Violin
  # ============================
  if (style == "violin") {
    g <- ggplot2::ggplot(data_long, ggplot2::aes(x = group, y = value, fill = group)) +
      ggplot2::geom_violin(
        trim = FALSE, alpha = .6,
        color = NA, adjust = .6
      ) +
      ggplot2::geom_boxplot(
        width = .18, outlier.shape = NA,
        color = "gray20", linewidth = .4
      ) +
      ggplot2::annotate(
        "text", x = mean(1:2), y = y_pos,
        label = signif_label, size = 6,
        col = "grey25"
      ) +
      ggplot2::geom_point(
        position = ggplot2::position_jitter(width = .1),
        alpha = .2, size = 1.8, color = "gray25"
      ) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::labs(
        title = title,
        subtitle = .build_subtitle_u(median_diff, p_value),
        x = "",
        y = ylab
      ) +
      ggplot2::theme(
        legend.position = "none",
        axis.text.x = ggplot2::element_text(
          angle = 45, hjust = 1, size = 12
        )
      )
  }

  # ============================
  # STYLE 3 — (Premium monochorme)
  # ============================
  if (style == "mono") {
    g <- ggplot2::ggplot(data_long, ggplot2::aes(x = group, y = value, fill = group)) +
      ggplot2::geom_boxplot(alpha = 0.75, outlier.shape = NA, width = 0.7, linewidth = 0.7) +
      ggplot2::geom_jitter(width = 0.1, alpha = 0.2, color = "grey25", size = 1.8) +
      ggplot2::annotate(
        "text", x = mean(1:2), y = y_pos,
        label = signif_label, size = 6,
        col = "grey25"
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::scale_fill_manual(values = mono_colors) +
      ggplot2::labs(
        title = title,
        subtitle = .build_subtitle_u(median_diff, p_value),
        x = "",
        y = ylab
      ) +
      ggplot2::theme(
        legend.position = "none",
        plot.margin = ggplot2::margin(5.5, 5.5, 10, 5.5),
        axis.text.x = ggplot2::element_text(
          angle = 45, hjust = 1, size = 12
        )
      )
  }

  # ============================
  # STYLE 4 — ggdist half-eye
  # ============================
  if (style == "halfeye") {

    if (!requireNamespace("ggdist", quietly = TRUE))
      stop("Package 'ggdist' is required for style = halfeye'.")

    g <- ggplot2::ggplot(
      data_long,
      ggplot2::aes(x = group, y = value, fill = group)
    ) +
      ggdist::stat_halfeye(
        alpha = 0.6,
        trim = FALSE,
        adjust = 0.6,
        width = 0.6,
        .width = c(0.5, 0.8, 0.95),
        justification = -0.2,
        slab_color = "gray20",
        interval_color = "gray20"
      ) +
      ggplot2::annotate(
        "text", x = mean(1:2), y = y_pos,
        label = signif_label, size = 6,
        col = "grey25"
      ) +
      ggplot2::geom_point(
        position = ggplot2::position_nudge(x = 0.15),
        size = 1.1, alpha = 0.4, color = "black"
      ) +
      ggdist::stat_pointinterval(
        position = ggplot2::position_nudge(x = 0.2),
        point_color = "black",
        interval_color = "black",
        .width = 0.95
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::labs(
        title = title,
        subtitle = .build_subtitle_u(median_diff, p_value),
        x = "",
        y = ylab
      ) +
      ggplot2::theme(
        legend.position = "none",
        axis.text.x = ggplot2::element_text(
          angle = 45, hjust = 1, size = 12
        )
      )
  }

  print(g)

  # ============================
  # Output
  # ============================
  summary_table <- data_long |>
    dplyr::group_by(group) |>
    dplyr::summarise(
      Median = round(stats::median(value, na.rm = TRUE), 2),
      Mean   = round(mean(value, na.rm = TRUE), 2),
      SD     = round(stats::sd(value, na.rm = TRUE), 2),
      .groups = "drop"
    )

  obj <- list(
    summary = summary_table,
    test    = test_result,
    plot    = g
  )

  # ============================
  # Return
  # ============================
  if (verbose) {

    .print_header("Mann-Whitney U Test")

    .print_block("Summary", function() {
      print(summary_table, row.names = FALSE)
    })

    .print_block("Statistics", function() {

      cat("W statistic:          ", round(test_result$statistic, 3), "\n", sep = "")
      cat("p-value:              ", p_label, "\n", sep = "")
      cat("Median difference:    ", round(median_diff, 2),
          " [", round(ci_low, 2), ", ", round(ci_high, 2), "]\n", sep = "")
      cat("Rank-biserial r:      ", round(r_rb, 3), "\n", sep = "")

    })
  }

  return(invisible(list(result = obj)))
}

#' Multiple pairwise comparisons with diagnostics
#'
#' Performs multiple pairwise comparisons using Student's t-test
#' or Mann-Whitney test, with automatic diagnostics, effect sizes,
#' confidence intervals, multiple testing correction and visualization.
#'
#' @param ... Numeric vectors or a data.frame with groups in columns.
#' @param comparisons List of character vectors specifying pairwise
#'   comparisons (e.g. list(c("A","B"), c("B","C"))).
#'   If NULL, all pairwise combinations are used.
#' @param title Plot title.
#' @param xlab X-axis label.
#' @param ylab Y-axis label.
#' @param style Plot style. One of "boxplot", "violin",
#'   "mono", or "halfeye".
#' @param p_adjust Method for multiple testing correction.
#'   One of "none", "holm", "BH", "bonferroni".
#' @param help Logical. If TRUE, prints usage examples.
#' @param verbose Logical. If TRUE, prints results and plots.
#'
#' @return A list with:
#' \describe{
#'   \item{results}{A tibble with test results.}
#'   \item{plot}{A ggplot object.}
#'   \item{data_long}{Long-format data used for plotting.}
#' }
#'
#' @details
#' Normality is assessed using Shapiro-Wilk tests and homogeneity
#' of variances using Levene's test. If assumptions are met, a
#' pooled-variance t-test is used. Otherwise, the Mann-Whitney test
#' is applied with bootstrap confidence intervals.
#'
#' Effect sizes:
#' \itemize{
#'   \item Cohen's d for t-tests
#'   \item Rank-biserial correlation for Mann-Whitney
#' }
#'
#' @examples
#' df <- data.frame(
#'   control   = rnorm(30, 10),
#'   treatment = rnorm(30, 12),
#'   test1     = rnorm(30, 11),
#'   test2     = rnorm(30, 15)
#' )
#'
#' test.tmulti(
#'   df,
#'   comparisons = list(
#'     c("control", "treatment"),
#'     c("treatment", "test1")
#'   )
#' )
#'
#' @export

test.tmulti <- function(...,
                        comparisons = NULL,
                        title = "Multiple comparisons (t / MW)",
                        xlab = "",
                        ylab = "Value",
                        style = c("boxplot", "violin", "mono", "halfeye"),
                        p_adjust = c("none", "holm", "BH", "bonferroni"),
                        help = FALSE,
                        verbose = TRUE) {

  # ==============================
  # Argument handling
  # ==============================

  style    <- match.arg(style)
  p_adjust <- match.arg(p_adjust)


  # ==============================
  # Quick help
  # ==============================

  if (help || length(list(...)) == 0) {

    message("
Function test.tmulti()

Performs multiple pairwise comparisons using
Student t-test or Mann-Whitney, with diagnostics.

Example:
df <- data.frame(
  control   = rnorm(30, 10),
  treatment = rnorm(30, 12),
  test1     = rnorm(30, 11),
  test2     = rnorm(30, 15)
)

test.tmulti(
  df,
  comparisons = list(c('control','treatment'),
                     c('treatment','test1'))
)
")

return(invisible(NULL))
  }

  # ==============================
  # Required packages
  # ==============================

  required_packages <- c(
    "ggplot2", "purrr", "tibble",
    "tidyr", "dplyr", "scales", "car"
  )

  for (pkg in required_packages) {

    if (!requireNamespace(pkg, quietly = TRUE)) {

      stop(
        sprintf("Package '%s' is required.", pkg),
        call. = FALSE
      )
    }
  }

  has_ggdist <- requireNamespace("ggdist", quietly = TRUE)


  # ==============================
  # Capture input
  # ==============================

  dots <- list(...)

  if (length(dots) == 1 && is.data.frame(dots[[1]])) {

    df <- as.data.frame(dots[[1]])
    group_order <- names(df)

  } else {

    group_values <- dots
    group_names  <- sapply(substitute(list(...))[-1], deparse)

    df <- as.data.frame(group_values)
    names(df) <- group_names

    group_order <- group_names
  }

  if (!all(sapply(df, is.numeric))) {
    stop("All columns must be numeric.")
  }


  # ==============================
  # Normalize comparisons
  # ==============================

  to_pairs <- function(x) {

    if (is.list(x) &&
        all(sapply(x, function(z)
          is.character(z) && length(z) == 2))) {

      return(x)
    }

    if (is.character(x) && length(x) == 2) {
      return(list(x))
    }

    if (is.character(x) && length(x) == 1 &&
        grepl("[-,;]", x)) {

      parts <- trimws(unlist(strsplit(x, "[-,;]+")))

      if (length(parts) == 2) {
        return(list(parts))
      }
    }

    if (is.character(x) && length(x) > 2) {

      if (length(x) %% 2 != 0) {
        stop("Character vector must have even length.")
      }

      return(split(x, rep(seq_along(x)/2, each = 2)))
    }

    stop("Invalid 'comparisons' format.")
  }


  if (is.null(comparisons)) {

    cmb <- combn(group_order, 2)
    comparisons <- split(t(cmb), seq_len(ncol(cmb)))

  } else {

    comparisons <- to_pairs(comparisons)
  }


  # ==============================
  # Long format
  # ==============================

  data_long <- tibble::as_tibble(df) |>
    tidyr::pivot_longer(
      cols = tidyselect::everything(),
      names_to  = "group",
      values_to = "value"
    )

  data_long$group <- factor(
    data_long$group,
    levels = group_order
  )


  # ==============================
  # Pair analysis
  # ==============================

  analyze_pair <- function(g1, g2) {

    v1 <- na.omit(df[[g1]])
    v2 <- na.omit(df[[g2]])


    # ------------------
    # Diagnostics
    # ------------------

    p_norm1 <- if (length(v1) >= 3)
      shapiro.test(v1)$p.value else NA

    p_norm2 <- if (length(v2) >= 3)
      shapiro.test(v2)$p.value else NA

    is_normal <- all(c(p_norm1, p_norm2) > 0.05,
                     na.rm = TRUE)


    lev <- tryCatch(
      car::leveneTest(
        value ~ group,
        data = data_long[
          data_long$group %in% c(g1, g2), ]
      ),
      error = function(e) NULL
    )

    is_homogeneous <-
      !is.null(lev) && lev$`Pr(>F)`[1] > 0.05

    # Decision
    use_t <- is_normal && is_homogeneous

    # ------------------
    # t-test
    # ------------------

    if (use_t) {

      res <- t.test(v1, v2, var.equal = TRUE)

      nx <- length(v1)
      ny <- length(v2)

      mean_diff <- mean(v1) - mean(v2)

      sd_pooled <- sqrt(
        ((nx - 1)*sd(v1)^2 +
           (ny - 1)*sd(v2)^2)/(nx + ny - 2)
      )

      d <- if (sd_pooled > 0)
        mean_diff / sd_pooled else NA


      return(
        tibble::tibble(

          group1 = g1,
          group2 = g2,

          test_type = "t",
          test_name = "Student t-test",

          estimate = mean_diff,

          ci_low  = res$conf.int[1],
          ci_high = res$conf.int[2],

          effect = d,

          p_value = res$p.value
        )
      )
    }

    # ------------------
    # Mann-Whitney
    # ------------------

    res <- wilcox.test(v1, v2, exact = FALSE)

    U <- as.numeric(res$statistic)

    nx <- length(v1)
    ny <- length(v2)

    r_rb <- 1 - (2*U)/(nx*ny)

    # ----------------------------
    # Bootstrap CI (median diff)
    # ----------------------------

    res_boot <- .boot_two_sample(
      v1,
      v2,
      stat_fun = function(a, b)
        median(a, na.rm = TRUE) - median(b, na.rm = TRUE)
    )

    ci_low  <- res_boot$ci_low
    ci_high <- res_boot$ci_high

    med_diff <- median(v1) - median(v2)


    tibble::tibble(

      group1 = g1,
      group2 = g2,

      test_type = "mw",
      test_name = "Mann-Whitney",

      estimate = med_diff,

      ci_low  = ci_low,
      ci_high = ci_high,

      effect = r_rb,

      p_value = res$p.value
    )
  }

  # ==============================
  # Run tests
  # ==============================

  results <- purrr::map_dfr(
    comparisons,
    ~ analyze_pair(.x[1], .x[2])
  )

  # ==============================
  # p-value adjustment
  # ==============================

  if (p_adjust != "none") {

    results$p_adj <- p.adjust(
      results$p_value,
      method = p_adjust
    )

  } else {

    results$p_adj <- results$p_value
  }

  # ==============================
  # Labels position
  # ==============================
  results$signif <- dplyr::case_when(
    results$p_adj < 0.001 ~ "***",
    results$p_adj < 0.01  ~ "**",
    results$p_adj < 0.05  ~ "*",
    TRUE ~ ""
  )

  results$x1 <- match(results$group1, group_order)
  results$x2 <- match(results$group2, group_order)

  y_max <- max(data_long$value, na.rm = TRUE)
  y_range <- diff(range(data_long$value, na.rm = TRUE))

  step <- 0.08 * y_range

  results$y <- y_max + seq_len(nrow(results)) * step

  results <- results |>
    dplyr::mutate(dist = abs(x1 - x2)) |>
    dplyr::arrange(dist)

  signif_pairs <- results |>
    dplyr::filter(signif != "")

  signif_pairs$y <- y_max + seq_len(nrow(signif_pairs)) * step

  # ==============================
  # Colors
  # ==============================
  # Vivid colors
  vivid_colors <- scales::hue_pal()(length(unique(data_long$group)))

  # mono
  n <- length(group_order)

  mono_colors <- gray.colors(
    n,
    start = 0.9,
    end = 0.1
  )

  # ==============================
  # Plot
  # ==============================
  base_theme <-
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      legend.position = "none",
      axis.text.x = ggplot2::element_text(
        angle = 45,
        hjust = 1,
        size = 12
      )
    )

  # ----------------------------
  # Style 1 (Boxplot)
  # ----------------------------
  if (style == "boxplot") {

    g <- ggplot2::ggplot(
      data_long,
      ggplot2::aes(group, value, fill = group)
    ) +
      ggplot2::geom_boxplot(alpha = 0.7, outlier.shape = NA) +
      ggplot2::geom_jitter(width = 0.1, alpha = 0.4) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      base_theme +
      ggplot2::labs(title = title, x = "", y = ylab)
    }

  # ----------------------------
  # Style 2 (Violin)
  # ----------------------------
  if (style == "violin") {

  g <- ggplot2::ggplot(
    data_long,
    ggplot2::aes(group, value, fill = group)
  ) +
    ggplot2::geom_violin(
      trim = FALSE,
      alpha = 0.55,
      color = NA,
      adjust = .6
    ) +
    ggplot2::geom_boxplot(
      width = 0.18,
      outlier.shape = NA
    ) +
    ggplot2::geom_point(
      position = ggplot2::position_jitter(width = .1),
      alpha = .4,
      size = 1.8,
      color = "gray25"
    ) +
    ggplot2::scale_fill_manual(values = vivid_colors) +
    base_theme +
    ggplot2::labs(title = title, x = "", y = ylab)
  }

  # ----------------------------
  # Style 3 (monochrome)
  # ----------------------------
  if (style == "mono") {

  g <- ggplot2::ggplot(
    data_long,
    ggplot2::aes(group, value, fill = group)
  ) +
    ggplot2::geom_boxplot(alpha = 0.7, outlier.shape = NA) +
    ggplot2::geom_jitter(width = 0.1, alpha = 0.4) +
    ggplot2::scale_fill_manual(values = mono_colors) +
    base_theme +
    ggplot2::labs(title = title, x = "", y = ylab)
  }

  # ----------------------------
  # Style 4 (Half eye)
  # ----------------------------
  if (style == "halfeye" && has_ggdist) {

  g <- ggplot2::ggplot(
    data_long,
    ggplot2::aes(group, value, fill = group)
  ) +
    ggdist::stat_halfeye(
      alpha = 0.6,
      trim = FALSE,
      adjust = 0.6,
      width = 0.6,
      .width = c(0.5, 0.8, 0.95),
      justification = -0.2,
      slab_color = "gray20",
      interval_color = "gray20"
    ) +
    ggplot2::geom_point(
      position = ggplot2::position_nudge(x = 0.15),
      size = 1.1,
      alpha = 0.4,
      color = "black"
    ) +
    ggdist::stat_pointinterval(
      position = ggplot2::position_nudge(x = 0.2),
      point_color = "black",
      interval_color = "black",
      .width = 0.95
    ) +
    ggplot2::scale_fill_manual(values = vivid_colors) +
    base_theme +
    ggplot2::labs(title = title, x = "", y = ylab)
  }

  # --------------------------
  # Annotation and print
  # --------------------------
  g <- g + .add_significance(signif_pairs, y_range)

  print(g)

  # ==============================
  # Output
  # ==============================
  obj <- (list(
    results   = results,
    plot      = g,
    data_long = data_long
  ))

  # ==============================
  # Return
  # ==============================
  if (verbose) {

    .print_header("Pairwise comparisons")

    .print_block("Results", function() {

      for (i in seq_len(nrow(results))) {

        r <- results[i, ]


        cat(
          r$group1, " vs ", r$group2,
          " (", r$test_name, ")\n",
          sep = ""
        )


        cat(
          "Estimate = ",
          round(r$estimate, 2),
          " [",
          round(r$ci_low, 2), ", ",
          round(r$ci_high, 2),
          "]\n",
          sep = ""
        )


        es_label <- if (r$test_type == "t") {
          "Cohen's d"
        } else {
          "Rank-biserial r"
        }

        cat(
          es_label,
          " = ",
          round(r$effect, 3),
          "\n",
          sep = ""
        )

        p_raw  <- r$p_value
        p_adjf <- r$p_adj

        p_fmt  <- .format_pval(p_raw)
        p_adjf <- .format_pval(p_adjf)

        if (p_adjust == "none") {

          cat(
            "p = ",
            .format_pval(r$p_value),
            "\n\n",
            sep = ""
          )

        } else {

          cat(
            "p = ",
            .format_pval(r$p_value),
            " (adj = ",
            .format_pval(r$p_adj),
            ", ",
            p_adjust,
            ")\n\n",
            sep = ""
          )
        }
      }
    })
  }

  return(invisible(list(result = obj)))
}

.onAttach <- function(libname, pkgname) {
  packageStartupMessage(
    "\n",
    crayon::green("autotestR "), "loaded successfully\n",
    "----------------------------------------\n",
    "Simplified statistical analysis with automatic diagnostics and clear graphics.\n",
    "GitHub: https://github.com/Luiz-Garcia-R/autotestR\n"
  )
}

#' Correlation Analysis with Automatic Method Selection
#'
#' Performs correlation analysis between two numeric variables using Pearson,
#' Spearman, or Kendall methods. When \code{method = "auto"}, the function
#' automatically selects the most appropriate method based on normality,
#' presence of ties, and proportion of outliers.
#'
#' In addition to hypothesis testing, the function provides diagnostic
#' information, effect size interpretation, and publication-ready
#' visualizations.
#'
#' @details
#' Method selection in automatic mode follows these rules:
#' \itemize{
#'   \item If more than 5\% of observations are classified as outliers,
#'         Kendall's tau is used.
#'   \item If both variables are normally distributed and no ties are present,
#'         Pearson's correlation is used.
#'   \item Otherwise, Spearman's rank correlation is applied.
#' }
#'
#' Normality is assessed using Shapiro-Wilk (n <= 50), Anderson-Darling
#' (50 < n <= 300), or Kolmogorov-Smirnov (n > 300) tests.
#'
#' @param x Numeric vector or data frame with exactly two numeric columns.
#' @param y Numeric vector (optional if \code{x} is a data frame).
#' @param method Correlation method: \code{"auto"} (default), \code{"pearson"},
#'   \code{"spearman"}, or \code{"kendall"}.
#' @param main Plot title. If \code{NULL}, an automatic title is generated.
#'   If \code{FALSE}, no title is displayed.
#' @param xlab Label for the x-axis. If \code{NULL}, a label is inferred
#'   from the input. If \code{FALSE}, no label is shown.
#' @param ylab Label for the y-axis. If \code{NULL}, a label is inferred
#'   from the input. If \code{FALSE}, no label is shown.
#' @param style Plot style: \code{"simple"} (default), \code{"inference"},
#'   \code{"structure"}, \code{"density"}, or \code{"distribution"}.
#' @param plot_normality Logical. If \code{TRUE}, displays QQ-plots for
#'   normality assessment.
#' @param help Logical. If \code{TRUE}, displays a detailed usage guide.
#' @param verbose Logical. If \code{TRUE}, prints information about method
#'   selection and diagnostics.
#'
#' @return
#' An object of class \code{"test.correlation"} containing:
#' \itemize{
#'   \item \code{call}: Matched function call.
#'   \item \code{data}: Input data (if \code{n <= 5000}).
#'   \item \code{n}: Sample size.
#'   \item \code{method}: Selected correlation method.
#'   \item \code{estimate}: Correlation coefficient.
#'   \item \code{effect}: Effect size information (r, rho, or tau, and r² when applicable).
#'   \item \code{conf.int}: Confidence interval.
#'   \item \code{p.value}: P-value of the test.
#'   \item \code{interpretation}: Qualitative interpretation of effect size.
#'   \item \code{diagnostics}: Normality, ties, and outlier diagnostics.
#'   \item \code{decision}: Automatic selection rationale.
#'   \item \code{htest}: Underlying \code{stats::cor.test} object.
#'   \item \code{plot}: \code{ggplot2} object.
#' }
#'
#' @seealso
#' \code{\link[stats]{cor.test}},
#' \code{\link[ggplot2]{ggplot}}
#'
#' @examples
#' # Pearson (approximately normal data)
#' set.seed(123)
#' x <- rnorm(50, sd = 0.1)
#' y <- x + rnorm(50, sd = 0.1)
#' test.correlation(x, y)
#'
#' # Spearman (non-normal data)
#' set.seed(123)
#' x <- runif(300)
#' y <- log(x + 0.1) + rnorm(300, sd = 0.5)
#' test.correlation(x, y)
#'
#' # Kendall (ties and outliers)
#' set.seed(123)
#' x <- runif(1000, 1, 100)
#' y <- sin(x) * 30 + rnorm(1000, 0, 10)
#' x[sample(1:500, 50)] <- 50
#' y[sample(1:500, 50)] <- 0
#' x_out <- runif(100, 10, 20)
#' y_out <- runif(100, 80, 120)
#' x <- c(x, x_out)
#' y <- c(y, y_out)
#' test.correlation(x, y)
#'
#' @export

test.correlation <- function(x,
                             y = NULL,
                             method = "auto",
                             main = NULL,
                             xlab = NULL,
                             ylab = NULL,
                             style = c("simple", "inference", "structure", "density", "distribution"),
                             plot_normality = FALSE,
                             help = FALSE,
                             verbose = TRUE
) {

  style <- match.arg(style)
  method <- match.arg(
    method,
    c("auto","pearson","spearman","kendall")
  )

  if (help || missing(x)) {
    if (verbose) message("
test.correlation() function

Description:
  Tests correlation between two numeric variables, automatically choosing
  between Pearson, Spearman and Kendall.
  Can display graphical normality diagnostics (QQ-plots).

Usage:
  test.correlation(x, y, method = 'auto', plot_normality = TRUE)

Arguments:
  x, y              Numeric vectors of equal length or a data frame with two numeric columns
  method            'auto', 'pearson', 'spearman' or 'kendall'
  verbose           If TRUE, shows which method was used and why
  help              If TRUE, displays this help message with examples
  plot_normality    If TRUE, displays QQ-plots

Examples:

# Pearson
  set.seed(123)
  x <- rnorm(50, sd = 0.1)
  y <- x + rnorm(50, sd = 0.1)
  test.correlation(x, y)

# Spearman
  set.seed(123)
  x <- runif(300)
  y <- log(x + 0.1) + rnorm(300, sd = 0.5)
  test.correlation(x, y)

# Kendall
  set.seed(123)
  x <- runif(1000, 1, 100)
  y <- sin(x) * 30 + rnorm(1000, 0, 10)
  x[sample(1:500, 50)] <- 50
  y[sample(1:500, 50)] <- 0
  x_out <- runif(100, 10, 20)
  y_out <- runif(100, 80, 120)
  x <- c(x, x_out)
  y <- c(y, y_out)
  test.correlation(x, y)
")
    return(invisible(NULL))
  }

  # ---------------------------
  # required packages
  # ---------------------------
  required_packages <- c("ggplot2", "ggExtra")
  lapply(required_packages, function(pkg) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
      stop(sprintf("Package '%s' is not installed.", pkg), call. = FALSE)
    }
  })

  # ---------------------------
  # Prepare data
  # ---------------------------
  if (is.data.frame(x)) {
    if (ncol(x) != 2) stop("The data frame must contain exactly two numeric columns.")
    if (!all(sapply(x, is.numeric))) stop("Both data frame columns must be numeric.")
    name_x <- colnames(x)[1]
    name_y <- colnames(x)[2]
    data_df <- data.frame(x = x[[1]], y = x[[2]])
  } else {
    if (is.null(y)) stop("Provide two numeric vectors or a data frame with two columns.")
    if (!is.numeric(x) || !is.numeric(y)) stop("Both vectors must be numeric.")
    if (length(x) != length(y)) stop("Vectors must have the same length.")
    name_x <- deparse(substitute(x))
    name_y <- deparse(substitute(y))
    data_df <- data.frame(x = x, y = y)
  }

  clean_label <- function(label) {
    if (is.null(label) || identical(label, FALSE)) return(label)
    label <- gsub(".*\\$", "", label)
    label <- gsub("_", " ", label)
    tools::toTitleCase(label)
  }

  name_x <- clean_label(name_x)
  name_y <- clean_label(name_y)

  # ---------------------------
  # Check and diagnostics
  # ---------------------------
  has_ties <- anyDuplicated(data_df$x) > 0 || anyDuplicated(data_df$y) > 0

  apply_normality_test <- function(v) {
    n <- length(v)

    if (n <= 50) {
      p <- stats::shapiro.test(v)$p.value
      method <- "Shapiro-Wilk"

    } else if (n <= 300) {
      if (!requireNamespace("nortest", quietly = TRUE)) {
        stop("Please install the 'nortest' package for Anderson-Darling.")
      }
      p <- nortest::ad.test(v)$p.value
      method <- "Anderson-Darling"

    } else {
      ks_res <- stats::ks.test(v, "pnorm", mean = mean(v), sd = sd(v))
      p <- ks_res$p.value
      method <- "Kolmogorov-Smirnov"
    }

    list(p = p, method = method)
  }

  norm_x <- apply_normality_test(data_df$x)
  norm_y <- apply_normality_test(data_df$y)

  normal_x <- norm_x$p > 0.05
  normal_y <- norm_y$p > 0.05

  if (plot_normality) {
    oldpar <- graphics::par(mfrow = c(1, 2))
    on.exit(graphics::par(oldpar), add = TRUE)
    stats::qqnorm(data_df$x, main = paste("QQ-Plot of", name_x))
    stats::qqline(data_df$x, col = "red")
    stats::qqnorm(data_df$y, main = paste("QQ-Plot of", name_y))
    stats::qqline(data_df$y, col = "red")
  }

  detect_outliers <- function(v) {
    q1 <- stats::quantile(v, 0.25)
    q3 <- stats::quantile(v, 0.75)
    iqr <- q3 - q1
    which(v < (q1 - 1.5 * iqr) | v > (q3 + 1.5 * iqr))
  }

  outlier_fraction <- max(
    length(detect_outliers(data_df$x)) / nrow(data_df),
    length(detect_outliers(data_df$y)) / nrow(data_df)
  )

  method_used <- if (method == "auto") {
    if (outlier_fraction > 0.05) {
      "kendall"
    } else if (normal_x && normal_y && !has_ties) {
      "pearson"
    } else {
      "spearman"
    }
  } else {
    method
  }

  if (verbose && method == "auto") {
    message(sprintf("Selected method: %s", method_used))

    if (!normal_x || !normal_y) message("- Data are not normally distributed")
    if (has_ties) message("- Ties detected in the data")
    if (outlier_fraction > 0.05)
      message(sprintf("- Presence of outliers detected: %.1f%%",
                      outlier_fraction * 100))
  }

  # ---------------------------
  # Statistical test
  # ---------------------------
  test <- stats::cor.test(data_df$x, data_df$y, method = method_used)
  coef_value <- round(unname(test$estimate), 3)
  p_val <- test$p.value

  interpretation <- cut(
    abs(coef_value),
    breaks = c(-Inf, 0.3, 0.5, 0.7, 0.9, Inf),
    labels = c("very weak or none", "weak", "moderate", "strong", "very strong"),
    right = FALSE
  )

  # Choose smoothing method for trend line
  smooth_method <- if (tolower(method_used) == "pearson") "lm" else "loess"

  auto_title <- paste(tools::toTitleCase(method_used), "Correlation")

  # Naming title and axis
  plot_title <- if (is.null(main)) {
    auto_title
  } else if (identical(main, FALSE)) {
    NULL
  } else {
    main
  }

  plot_xlab <- if (is.null(xlab)) {
    name_x
  } else if (identical(xlab, FALSE)) {
    NULL
  } else {
    xlab
  }

  plot_ylab <- if (is.null(ylab)) {
    name_y
  } else if (identical(ylab, FALSE)) {
    NULL
  } else {
    ylab
  }

  # --- Subtitle text ---
  subtitle_text <- .make_subtitle_correlation(
    method_used,
    coef_value,
    nrow(data_df),
    p_val
  )

  # ---------------------------
  # Plot styles
  # ---------------------------
  if (style == "simple") {
    g <- ggplot2::ggplot(data_df, ggplot2::aes(x = x, y = y)) +
      ggplot2::geom_point(alpha = 0.40, size = 2.5, col = "grey40") +
      ggplot2::geom_smooth(method = smooth_method, se = FALSE, linetype = "dashed") +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::labs(
        title = plot_title,
        subtitle = subtitle_text,
        x = plot_xlab,
        y = plot_ylab
      )

  } else if (style == "inference") {
    line_col <- "#2F5D8A"
    ci_fill  <- "#2F5D8A"
    point_col <- "grey40"

    g <- ggplot2::ggplot(data_df, ggplot2::aes(x = x, y = y)) +
      ggplot2::geom_point(alpha = 0.40, size = 2.3, color = point_col) +
      ggplot2::geom_smooth(
        method = "lm",
        se = TRUE,
        level = 0.95,
        color = line_col,
        fill = ci_fill,
        alpha = 0.25,
        linewidth = 1.1
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::labs(
        title = plot_title,
        subtitle = subtitle_text,
        x = plot_xlab,
        y = plot_ylab
      )

  } else if (style == "structure") {
    g <- ggplot2::ggplot(data_df, ggplot2::aes(x = x, y = y)) +
      ggplot2::geom_point(alpha = 0.25, size = 2) +
      ggplot2::geom_density_2d(color = "grey40", alpha = 0.7) +
      ggplot2::geom_smooth(method = smooth_method, se = FALSE) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::labs(
        title = plot_title,
        subtitle = subtitle_text,
        x = plot_xlab,
        y = plot_ylab
      )

  } else if (style == "density") {
    g <- ggplot2::ggplot(data_df, ggplot2::aes(x = x, y = y)) +
      ggplot2::geom_hex() +
      ggplot2::scale_fill_gradientn(
        colours = c("#0000FF", "#00FF00", "#FFFF00", "#FF0000"),
        trans = "sqrt",
        name = "Density"
      ) +
      ggplot2::geom_smooth(method = smooth_method, se = FALSE, color = "white") +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::labs(
        title = plot_title,
        subtitle = subtitle_text,
        x = plot_xlab,
        y = plot_ylab
      )

  } else if (style == "distribution") {

    if (!requireNamespace("ggExtra", quietly = TRUE)) {
      warning("For style = 'distribution' install the 'ggExtra' package. Falling back to style 'simple'.")
      g <- ggplot2::ggplot(data_df, ggplot2::aes(x = x, y = y)) +
        ggplot2::geom_point(alpha = 0.25, size = 2.5) +
        ggplot2::geom_smooth(method = smooth_method, se = FALSE, linetype = "dashed") +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::labs(
          title = plot_title,
          subtitle = subtitle_text,
          x = plot_xlab,
          y = plot_ylab
        )

    } else {
      base_plot <- ggplot2::ggplot(data_df, ggplot2::aes(x = x, y = y)) +
        ggplot2::geom_point(alpha = 0.40, size = 2, col = "grey40") +
        ggplot2::geom_smooth(method = smooth_method, se = FALSE) +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::labs(
          title = plot_title,
          subtitle = subtitle_text,
          x = plot_xlab,
          y = plot_ylab
        )
      g <- ggExtra::ggMarginal(base_plot, type = "density", size = 5, fill = "#c8d3e0")
    }

  } else {
    stop("Invalid style: use 'simple', 'inference', 'structure', 'density' or 'distribution'")
  }

  # ---------------------------
  # Return
  # ---------------------------
  out <- list(
    call = match.call(),
    data = if (nrow(data_df) <= 5000) data_df else NULL,
    n = nrow(data_df),
    method = method_used,
    estimate = coef_value,
    conf.int = test$conf.int,
    p.value = test$p.value,
    interpretation = interpretation,
    effect = list(
      statistic = switch(
        method_used,
        pearson  = "r",
        spearman = "rho",
        kendall  = "tau"
      ),
      value = coef_value,
      r2 = if (method_used == "pearson") coef_value^2 else NULL
    ),
    diagnostics = list(
      normal_x = normal_x,
      normal_y = normal_y,
      ties = has_ties,
      outlier_fraction = outlier_fraction
    ),
    decision = list(
      auto = (method == "auto"),
      chosen = method_used,
      reasons = Filter(
        Negate(is.null),
        list(
          if (!normal_x || !normal_y) "non-normality",
          if (has_ties) "ties",
          if (outlier_fraction > 0.05) "outliers"
        )
      )
    ),
    htest = test,
    plot = g,
    verbose = verbose
  )

  class(out) <- "test.correlation"
  out
}

# ---------------------------
# S3 Output
# ---------------------------
#' @method print test.correlation
#' @param ... Additional arguments passed to other print methods (currently ignored)
#' @export
#' @rdname test.correlation

print.test.correlation <- function(x, ...) {

  if (!isTRUE(x$verbose)) return(invisible(x))

  .print_header("Correlation analysis")

  .print_block("Method", function() {
    cat("Method:", x$method, "\n")
    cat(
      "Correlation (", x$effect$statistic, "): ",
      round(x$effect$value, 3),
      "\n",
      sep = ""
    )
    cat("p-value:", .format_pval(x$p.value), "\n")
  })

  .print_block("Diagnostics", function() {
    cat("Normal X:", x$diagnostics$normal_x, "\n")
    cat("Normal Y:", x$diagnostics$normal_y, "\n")
    cat("Ties:", x$diagnostics$ties, "\n")
    cat("N:", x$n, "\n")
    cat("Outliers:", round(x$diagnostics$outlier_fraction * 100, 1), "%\n")
  })

  if (!is.null(x$plot)) print(x$plot)

  invisible(x)
}

# =============================================================================
# Auxiliary stats functions
# =============================================================================

# =====================================
# Format p_val
# =====================================
#' @keywords internal
.format_pval <- function(p, digits = 3, sci_cut = 1e-3) {

  if (is.na(p)) return(NA_character_)

  if (p < .Machine$double.eps) {
    return("< 2.2e-16")
  }

  if (p < sci_cut) {
    return(formatC(p, format = "e", digits = 2))
  }

  round_p <- round(p, digits)

  formatC(round_p, format = "f", digits = digits)
}

# =====================================
# Compute cramers_v
# =====================================
#' @keywords internal
.cramers_v <- function(tab) {

  chi <- suppressWarnings(chisq.test(tab, correct = FALSE))

  n <- sum(tab)

  r <- nrow(tab)
  c <- ncol(tab)

  v <- sqrt(
    as.numeric(chi$statistic) /
      (n * min(r - 1, c - 1))
  )

  v
}

# =====================================
# Compute odds_ratio_ci
# =====================================
#' @keywords internal
.odds_ratio_ci <- function(tab, conf = 0.95) {

  if (!all(dim(tab) == c(2,2)))
    stop("Odds ratio only for 2x2 tables.")

  a <- tab[1,1]
  b <- tab[1,2]
  c <- tab[2,1]
  d <- tab[2,2]

  # Haldane correction
  if (any(tab == 0)) {
    a <- a + 0.5
    b <- b + 0.5
    c <- c + 0.5
    d <- d + 0.5
  }

  or <- (a * d) / (b * c)

  se <- sqrt(1/a + 1/b + 1/c + 1/d)

  z <- qnorm(1 - (1 - conf)/2)

  low <- exp(log(or) - z * se)
  high <- exp(log(or) + z * se)

  list(
    or = or,
    ci_low = low,
    ci_high = high
  )
}

# =====================================
# Add significance
# =====================================
#' @keywords internal
.add_significance <- function(sig_pairs, y_range, text_size = 5) {

  if (is.null(sig_pairs) || nrow(sig_pairs) == 0) {
    return(NULL)
  }

  list(

    ggplot2::geom_segment(
      data = sig_pairs,
      ggplot2::aes(
        x = x1,
        xend = x2,
        y = y,
        yend = y
      ),
      inherit.aes = FALSE
    ),

    ggplot2::geom_segment(
      data = sig_pairs,
      ggplot2::aes(
        x = x1,
        xend = x1,
        y = y,
        yend = y - 0.02 * y_range
      ),
      inherit.aes = FALSE
    ),

    ggplot2::geom_segment(
      data = sig_pairs,
      ggplot2::aes(
        x = x2,
        xend = x2,
        y = y,
        yend = y - 0.02 * y_range
      ),
      inherit.aes = FALSE
    ),

    ggplot2::geom_text(
      data = sig_pairs,
      ggplot2::aes(
        x = (x1 + x2) / 2,
        y = y + 0.02 * y_range,
        label = signif
      ),
      inherit.aes = FALSE,
      size = text_size
    )
  )
}

# =====================================
# P value format
# =====================================
#' @keywords internal
.format_p <- function(p) {

  if (is.na(p)) return("= NA")

  if (p < 0.001) {
    "< 0.001"
  } else {
    paste0("= ", formatC(p, format = "f", digits = 3))
  }
}

# =====================================
# U test subtitle
# =====================================
#' @keywords internal
.build_subtitle_u <- function(median_diff, p_value) {

  p_txt <- .format_p(p_value)

  if (p_value < 0.001) {
    paste0("med diff = ", round(median_diff, 2), " | p < 0.001")
  } else {
    paste0("med diff = ", round(median_diff, 2), " | p = ", p_txt)
  }
}

# =====================================
# Paired T test subtitle
# =====================================
#' @keywords internal
.build_subtitle <- function(effect, p_value, label = "diff") {

  p_txt <- .format_p(p_value)

  if (p_value < 0.001) {
    paste0(label, " = ", round(effect, 2), " | p < 0.001")
  } else {
    paste0(label, " = ", round(effect, 2), " | p = ", p_txt)
  }
}

# =====================================
# Kruskal Wallis subtitle
# =====================================
#' @keywords internal
.build_subtitle_kw <- function(p_value, eps) {

  p_txt <- .format_p(p_value)

  if (p_value < 0.001) {
    paste0(
      "eps^2 = ", round(eps, 2),
      " | p < 0.001"
    )
  } else {
    paste0(
      "eps^2 = ", round(eps, 2),
      " | p = ", p_txt
    )
  }
}

# =====================================
# ANOVA subtitle
# =====================================
#' @keywords internal
.make_subtitle_anova <- function(omega_sq, p_value) {

  p_label <- .format_p(p_value)

  paste0(
    "omega^2 = ", round(omega_sq, 2),
    " | p ", p_label
  )
}

# =====================================
# Interaction subtitle
# =====================================
#' @keywords internal
.build_subtitle_interaction <- function(beta, p_value) {

  p_txt <- .format_p(p_value)

  if (p_value < 0.001) {
    paste0(
      "slope diff = ", round(beta, 2),
      " | p < 0.001"
    )
  } else {
    paste0(
      "slope diff = ", round(beta, 2),
      " | p = ", p_txt
    )
  }
}

# =====================================
# Correlation subtitle
# =====================================
#' @keywords internal
.make_subtitle_correlation <- function(method, estimate, n, p_value) {

  stat_name <- switch(
    method,
    pearson  = "r",
    spearman = "rho",
    kendall  = "tau"
  )

  p_label <- .format_p(p_value)

  paste0(
    stat_name, " = ", round(estimate, 2),
    " | p ", p_label,
    " (n=", n, ")"
  )
}

# =====================================
# Chi-square subtitle
# =====================================
#' @keywords internal
.make_subtitle_chi <- function(cramers_v, p_value, small_expected = FALSE) {

  p_label <- .format_p(p_value)

  base <- paste0(
    "V = ", round(cramers_v, 2),
    " | p ", p_label
  )

  if (small_expected) {
    base <- paste0(base, " | low expected counts")
  }

  base
}

# =====================================
# Fisher subtitle
# =====================================
#' @keywords internal
.make_subtitle_fisher <- function(or, p_value) {

  p_label <- .format_p(p_value)

  or_label <- if (is.infinite(or)) {
    "Inf"
  } else {
    round(or, 2)
  }

  paste0(
    "OR = ", or_label,
    " | p ", p_label
  )
}

#' Student's t-test (with automatic diagnostics)
#'
#' Performs Student's t-test to compare means between two independent groups,
#' with automatic checks for normality and homogeneity of variances.
#' If assumptions are violated, the Mann-Whitney test is automatically applied
#' (without generating a plot).
#'
#' @param ... Two numeric vectors or a data frame with exactly two columns.
#' @param title Logical. If true, return a plot entitled.
#' @param title_text Plot title (string). Default: "t-test".
#' @param xlab X-axis label in the plot (string). Default: "Group".
#' @param ylab Y-axis label in the plot (string). Default: "Value".
#' @param style Plot aesthetic generated by the function.
#' @param help Logical. If TRUE, shows a detailed explanation of the function.
#'        Default: FALSE.
#' @param verbose Logical. If TRUE, prints detailed messages. Default: TRUE.
#' @return Invisible list with summary, test test_result, method and (optionally) plot.
#' @export
#'
#' @examples
#' set.seed(123)
#' df <- data.frame(
#'   control = rnorm(30, 10),
#'   treatment = rnorm(30, 15)
#' )
#' test.t(df)

test.t <- function(...,
                   title = TRUE,
                   title_text = "t-test" ,
                   xlab = "Group",
                   ylab = "Value",
                   style = c("boxplot", "violin", "mono", "halfeye"),
                   help = FALSE,
                   verbose = TRUE) {

  args <- list(...)
  style <- match.arg(style)

  # ============================
  # Quick help
  # ============================
  if (help) {
    if (verbose) {
      message("
Function test.t()

Description:
  Performs Student's t-test to compare the means of two independent groups,
  with automatic checks for normality (Shapiro-Wilk) and homogeneity of variances
  (Levene's test).

If assumptions are violated, the Mann-Whitney test is automatically applied
and no plot is generated.

Accepted inputs:
  - Two numeric vectors (e.g., group1, group2)
  - A data frame with exactly two numeric columns

Example:
  set.seed(123)
  df <- data.frame(
    control = rnorm(30, 10),
    treatment = rnorm(30, 15)
  )
  test.t(df)
")
    }
    return(invisible(NULL))
  }

  # ============================
  # Detect data frame input
  # ============================
  if (length(args) == 1 && is.data.frame(args[[1]])) {
    df <- args[[1]]
    if (ncol(df) != 2) {
      stop("The data frame must contain exactly two numeric columns.")
    }
    groups <- as.list(df)
    group_names <- colnames(df)
  } else {
    groups <- args
    if (length(groups) != 2) {
      stop("Provide exactly two groups or a data frame with two columns.")
    }
    group_names <- sapply(substitute(list(...))[-1], deparse)
  }

  # ============================
  # Validation
  # ============================
  if (!all(sapply(groups, is.numeric))) {
    stop("Both groups must be numeric.")
  }
  if (any(sapply(groups, function(g) sd(g, na.rm = TRUE) == 0))) {
    stop("One of the groups has zero variance (constant values).")
  }

  # ============================
  # Required packages
  # ============================
  required_packages <- c("ggplot2", "dplyr", "car", "ggdist")
  for (pkg in required_packages) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
      stop(
        paste0(
          "Package '", pkg,
          "' is not installed. Install it with install.packages('", pkg, "')"
        )
      )
    }
  }

  # ============================
  # Build long-format data frame
  # ============================
  values <- unlist(groups)
  group <- factor(
    rep(group_names, times = sapply(groups, length)),
    levels = group_names
  )
  data <- data.frame(value = values, group = group)

  # ============================
  # Normality test (Shapiro-Wilk)
  # ============================
  normality <- sapply(groups, function(g) {
    if (length(g) < 3) return(NA)
    stats::shapiro.test(g[!is.na(g)])$p.value
  })
  is_normal <- all(normality > 0.05, na.rm = TRUE)

  # ============================
  # Homogeneity test (Levene)
  # ============================
  homogeneity <- tryCatch(
    car::leveneTest(value ~ group, data = data, na.action = na.omit),
    error = function(e) data.frame(`Pr(>F)` = NA)
  )
  is_homogeneous <- !is.na(homogeneity$`Pr(>F)`[1]) &&
    homogeneity$`Pr(>F)`[1] > 0.05

  # --- Assumption checks ---
  violated_normality   <- !is_normal
  violated_homogeneity <- !is_homogeneous

  assumption_warning <- violated_normality || violated_homogeneity

  # ============================
  # Student t-test
  # ============================
  test_result <- stats::t.test(groups[[1]], groups[[2]])

  method <- "Student's t-test"

  ci_low  <- test_result$conf.int[1]
  ci_high <- test_result$conf.int[2]

  p_value <- test_result$p.value
  p_label <- .format_pval(p_value)

  # ============================
  # Effect size: Cohen's d
  # ============================
  x <- groups[[1]]
  y <- groups[[2]]

  nx <- sum(!is.na(x))
  ny <- sum(!is.na(y))

  mean_diff <- mean(x, na.rm = TRUE) - mean(y, na.rm = TRUE)

  sd_pooled <- sqrt(
    ((nx - 1) * sd(x, na.rm = TRUE)^2 +
       (ny - 1) * sd(y, na.rm = TRUE)^2) /
      (nx + ny - 2)
  )

  cohen_d <- mean_diff / sd_pooled

  # ============================
  # Descriptive summary
  # ============================
  summary_table <- data |>
    dplyr::group_by(group) |>
    dplyr::summarise(
      Mean = round(mean(value, na.rm = TRUE), 2),
      SD = round(sd(value, na.rm = TRUE), 2),
      .groups = "drop"
    )

  if (verbose && assumption_warning) {

    .print_block("Assumption check", function() {

      if (violated_normality) {
        cat("Warning: Normality assumption violated (Shapiro-Wilk p < 0.05)\n")
      }

      if (violated_homogeneity) {
        cat("Warning: Variance homogeneity violated (Levene p < 0.05)\n")
      }

      cat("Interpret results with caution.\n")

    })
  }

  # ============================
  # Labels and colors
  # ============================
    p_label <- signif(p_value, 3)

    p_label_sub <- .format_p(p_value)

    signif_label <- if (p_value < 0.001) {
      "***"
    } else if (p_value < 0.01) {
      "**"
    } else if (p_value < 0.05) {
      "*"
    } else {
      ""
    }

    # --- Subtitle ---
    subtitle_text <- paste0(
      "diff = ", round(mean_diff, 2),
      " | p ", ifelse(p_value < 0.001, "", "= "), p_label_sub
    )

    # --- Colors ---
    y_pos <- max(values, na.rm = TRUE) +
      0.1 * diff(range(values, na.rm = TRUE))

    vivid_colors <- scales::hue_pal()(length(unique(data$group)))
    mono_colors <- c("grey75", "grey25")

  # ============================
  # STYLE 1 (Boxplot + jitter)
  # ============================
    if (style == "boxplot") {
      g <- ggplot2::ggplot(data, ggplot2::aes(x = group, y = value, fill = group)) +
        ggplot2::geom_boxplot(alpha = 0.75, outlier.shape = NA, width = 0.7, linewidth = 0.7) +
        ggplot2::geom_jitter(width = 0.1, alpha = 0.2, color = "grey25", size = 1.8) +
        ggplot2::annotate(
          "text", x = mean(1:2), y = y_pos,
          label = signif_label, size = 6,
          col = "grey25"
        ) +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::scale_fill_manual(values = vivid_colors) +
        ggplot2::labs(
          title = if (title) title_text else NULL,
          subtitle = subtitle_text,
          x = "",
          y = ylab
        ) +
        ggplot2::theme(
          legend.position = "none",
          plot.margin = ggplot2::margin(5.5, 5.5, 10, 5.5),
          axis.text.x = ggplot2::element_text(
            angle = 45, hjust = 1, size = 12
          )
        )
    }

  # ============================
  # STYLE 2 (Clean violin)
  # ============================
    if (style == "violin") {
      g <- ggplot2::ggplot(data, ggplot2::aes(x = group, y = value, fill = group)) +
        ggplot2::geom_violin(
          trim = FALSE, alpha = .6,
          color = NA, adjust = .6
        ) +
        ggplot2::geom_boxplot(
          width = .18, outlier.shape = NA,
          color = "gray20", linewidth = .4
        ) +
        ggplot2::annotate(
          "text", x = mean(1:2), y = y_pos,
          label = signif_label, size = 6,
          col = "grey25"
        ) +
        ggplot2::geom_point(
          position = ggplot2::position_jitter(width = .1),
          alpha = .2, size = 1.8, color = "gray25"
        ) +
        ggplot2::scale_fill_manual(values = vivid_colors) +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::labs(
          title = if (title) title_text else NULL,
          subtitle = subtitle_text,
          x = "",
          y = ylab
        ) +
        ggplot2::theme(
          legend.position = "none",
          axis.text.x = ggplot2::element_text(
            angle = 45, hjust = 1, size = 12
          )
        )
    }

  # ============================
  # STYLE 3 (Premium monochrome)
  # ============================
  if (style == "mono") {
    g <- ggplot2::ggplot(data, ggplot2::aes(x = group, y = value, fill = group)) +
      ggplot2::geom_boxplot(alpha = 0.75, outlier.shape = NA, width = 0.7, linewidth = 0.7) +
      ggplot2::geom_jitter(width = 0.1, alpha = 0.2, color = "grey25", size = 1.8) +
      ggplot2::annotate(
        "text", x = mean(1:2), y = y_pos,
        label = signif_label, size = 6,
        col = "grey25"
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::scale_fill_manual(values = mono_colors) +
      ggplot2::labs(
        title = if (title) title_text else NULL,
        subtitle = subtitle_text,
        x = "",
        y = ylab
      ) +
      ggplot2::theme(
        legend.position = "none",
        plot.margin = ggplot2::margin(5.5, 5.5, 10, 5.5),
        axis.text.x = ggplot2::element_text(
          angle = 45, hjust = 1, size = 12
        )
      )
  }

  # ============================
  # STYLE 4 (ggdist half-eye + median)
  # ============================
    if (style == "halfeye") {

      if (!requireNamespace("ggdist", quietly = TRUE))
        stop("Package 'ggdist' is required for style = halfeye'.")

      g <- ggplot2::ggplot(
        data,
        ggplot2::aes(x = group, y = value, fill = group)
      ) +
        ggdist::stat_halfeye(
          alpha = 0.6,
          trim = FALSE,
          adjust = 0.6,
          width = 0.6,
          .width = c(0.5, 0.8, 0.95),
          justification = -0.2,
          slab_color = "gray20",
          interval_color = "gray20"
        ) +
        ggplot2::annotate(
          "text", x = mean(1:2), y = y_pos,
          label = signif_label, size = 6,
          col = "grey25"
        ) +
        ggplot2::geom_point(
          position = ggplot2::position_nudge(x = 0.15),
          size = 1.1, alpha = 0.4, color = "black"
        ) +
        ggdist::stat_pointinterval(
          position = ggplot2::position_nudge(x = 0.2),
          point_color = "black",
          interval_color = "black",
          .width = 0.95
        ) +
        ggplot2::theme_minimal(base_size = 12) +
        ggplot2::scale_fill_manual(values = vivid_colors) +
        ggplot2::labs(
          title = if (title) title_text else NULL,
          subtitle = subtitle_text,
          x = "",
          y = ylab
        ) +
        ggplot2::theme(
          legend.position = "none",
          axis.text.x = ggplot2::element_text(
            angle = 45, hjust = 1, size = 12
          )
        )
    }

    print(g)

  # ============================
  # Output
  # ============================
  obj <- list(
    summary = summary_table,
    result = test_result,
    method = method,
    normality = normality,
    homogeneity = homogeneity,
    plot = g
  )

  # ============================
  # Return
  # ============================
  if (verbose) {

    .print_header(method)

    .print_block("Summary", function() {
      print(summary_table, row.names = FALSE)
    })

    .print_block("Statistics", function() {

      cat("t statistic:          ", round(test_result$statistic, 3), "\n",
          "Degrees of freedon:   ", round(test_result$parameter, 1), "\n",
          "p-value:              ", p_label, "\n", sep = "")
      cat("Mean difference:      ", round(mean_diff, 2), " [", round(ci_low, 2), ", ", round(ci_high, 2), "]\n", sep = "")
      cat("Cohen's d:            ", round(cohen_d, 3), "\n", sep = "")

    })
  }

  return(invisible(list(result = obj)))

}

#' autotestR
#'
#' designed to simplify the execution of the main statistical tests commonly used in the life sciences.
#' It provides user-friendly functions that automatically generate plots and clear explanations,
#' making statistical analysis more accessible for researchers and students.
#'
#' # Main features
#'
#' - t test (independent and paired)
#' - Mann–Whitney test (Wilcoxon rank-sum)
#' - Multiple group comparison (t test or Mann–Whitney)
#' - Chi-squared test and Fisher’s exact test
#' - One-way ANOVA with Tukey HSD post hoc test
#' - Kruskal–Wallis test with Dunn post hoc test
#' - Pearson, Spearman, and Kendall correlation tests with automatic plots
#' - Diagnostic function that suggests the most appropriate statistical test
#' - Intuitive plots fully integrated into the functions
#'
#' ### Basic usage
#'
#' library(autotestR)
#'
#' # Independent t test
#' group1 <- rnorm(30, 10, 2)
#' group2 <- rnorm(30, 12, 2)
#' test.t(group1, group2)
#'
#' # Chi-squared test
#' var1 <- sample(c("A", "B"), 100, replace = TRUE)
#' var2 <- sample(c("Yes", "No"), 100, replace = TRUE)
#' test.chi(var1, var2)
#'
#' # Multiple test (t test or Mann–Whitney)
#' df <- data.frame(
#'  control   = rnorm(30, 10),
#'  treatment = rnorm(30, 12),
#'  test1     = rnorm(30, 11),
#'  test2     = rnorm(30, 15)
#' )
#' test.tmulti(df)
#'
#' # ANOVA with post hoc test
#' g1 <- rnorm(20, 5)
#' g2 <- rnorm(20, 7)
#' g3 <- rnorm(20, 6)
#' test.anova(g1, g2, g3)
#'
#' # Correlation test
#' x <- rnorm(30)
#' y <- x + rnorm(30, 0, 1)
#' test.correlation(x, y)
#'
#' @name autotestR
#'
"_PACKAGE"

#' ANOVA test with automated assumption checking
#'
#' Performs ANOVA (and Tukey HSD) if data meet normality and homogeneity assumptions.
#' Otherwise, automatically recommends Kruskal-Wallis/Dunn.
#'
#' @param ... Vectors or a data.frame with >= 2 columns.
#' @param title Plot title.
#' @param xlab X-axis label.
#' @param ylab Y-axis label.
#' @param style Aesthetic style of the generated plot.
#' @param help If TRUE, shows help.
#' @param verbose If TRUE, shows detailed messages.
#' @importFrom stats aov sd aggregate shapiro.test var.test
#' @return An `aov` object or a recommendation message.
#' @export
#'
#' @examples
#' df <- data.frame(
#'   control = rnorm(30, 10),
#'   treatment = rnorm(30, 12),
#'   test = rnorm(30, 11)
#' )
#' test.anova(df)

test.anova <- function(...,
                       title = "ANOVA/Tukey HSD",
                       xlab = "Group",
                       ylab = "Value",
                       style = c("boxplot", "violin", "mono", "halfeye"),
                       help = FALSE,
                       verbose = TRUE) {

  style <- match.arg(style)

  # --- Quick help block ---
  if (help || length(list(...)) == 0) {
    message("
test.anova() function

Description:
  Performs ANOVA between numeric groups, followed by Tukey HSD,
  if normality and homogeneity assumptions are met.
  Otherwise, recommends Kruskal-Wallis/Dunn.

Example:
  df <- data.frame(
    control = rnorm(30, 10, sd = 0.5),
    treatment = rnorm(30, 11, sd = 0.5),
    test = rnorm(30, 11, sd = 0.5)
  )

test.anova(df)
")
    return(invisible(NULL))
  }

  required_packages <- c("ggplot2", "car")
  for (pkg in required_packages) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
      stop(paste0("Please install package: ", pkg))
    }
  }

  dots <- list(...)
  if (length(dots) == 1 && is.data.frame(dots[[1]])) {
    df <- dots[[1]]
    groups <- as.list(df)
    group_names <- names(df)
  } else {
    groups <- dots
    group_names <- sapply(substitute(list(...))[-1], deparse)
  }

  values <- unlist(groups)
  group_factor <- factor(rep(group_names, times = sapply(groups, length)))
  data_long <- data.frame(value = values, group = group_factor)

  # -------------------------
  # Normality test
  # -------------------------
  apply_normality_test <- function(x) {

    out <- tryCatch(
      shapiro.test(x)$p.value,
      error = function(e) NA
    )

    out
  }

  p_normal <- sapply(groups, apply_normality_test)

  normal <- all(!is.na(p_normal) & p_normal > 0.05)

  p_levene <- tryCatch({
    if (length(groups) > 2) {
      car::leveneTest(value ~ group, data = data_long)$`Pr(>F)`[1]
    } else {
      var.test(groups[[1]], groups[[2]])$p.value
    }
  }, error = function(e) NA)
  homogeneous <- !is.na(p_levene) && p_levene > 0.05

  # -------------------------
  # If assumptions fail
  # -------------------------
  if (!normal || !homogeneous) {
    if (verbose) message("\nAssumptions failed. Recommendation: Kruskal-Wallis/Dunn")

    means_sd <- aggregate(value ~ group, data = data_long,
                          function(x) c(mean = mean(x), sd = sd(x)))
    means_sd <- do.call(data.frame, means_sd)
    colnames(means_sd)[2:3] <- c("mean", "sd")

    return(invisible(list(
      type = "ANOVA - assumptions not met",
      normal = normal,
      p_normal = p_normal,
      homogeneous = homogeneous,
      p_levene = p_levene,
      recommendation = "Use Kruskal-Wallis/Dunn",
      means_sd = means_sd
    )))
  }

  # -------------------------
  # ANOVA
  # -------------------------
  model <- aov(value ~ group, data = data_long)
  p_anova <- summary(model)[[1]][["Pr(>F)"]][1]

  # --- Effect size: Omega squared (ω²) ---
  anova_tab <- summary(model)[[1]]

  ss_between <- anova_tab[1, "Sum Sq"]
  ss_within  <- anova_tab[nrow(anova_tab), "Sum Sq"]

  df_between <- anova_tab[1, "Df"]
  ms_within  <- anova_tab[nrow(anova_tab), "Mean Sq"]

  ss_total <- ss_between + ss_within

  omega_sq <- (ss_between - df_between * ms_within) /
    (ss_total + ms_within)

  F_value <- anova_tab[1, "F value"]
  df1 <- anova_tab[1, "Df"]
  df2 <- anova_tab[nrow(anova_tab), "Df"]
  n_total <- nrow(data_long)

  # --- Bootstrap CI for omega squared ---
  boot_omega <- .boot_anova_omega(
    data  = data_long,
    group = "group",
    value = "value",
    B = 2000
  )

  omega_ci_low  <- boot_omega$ci_low
  omega_ci_high <- boot_omega$ci_high

  # Tukey table
  tukey_res <- TukeyHSD(model)$group
  tukey_df <- as.data.frame(tukey_res)

  # Significant Tukey pairs (p < 0.05)
  tukey_pairs <- data.frame(
    Comparison = rownames(tukey_res),
    diff = tukey_res[, "diff"],
    lwr  = tukey_res[, "lwr"],
    upr  = tukey_res[, "upr"],
    p_adj = tukey_res[, "p adj"],
    stringsAsFactors = FALSE
  )

  significant_pairs <- subset(tukey_pairs, p_adj < 0.05)

  tukey_df$Comparison <- rownames(tukey_res)
  rownames(tukey_df) <- NULL

  # Means and SD
  means_sd <- aggregate(value ~ group, data = data_long,
                        function(x) c(mean = mean(x), sd = sd(x)))
  means_sd <- do.call(data.frame, means_sd)
  colnames(means_sd)[2:3] <- c("mean", "sd")

  # --------------------------
  # Labels position
  # --------------------------
  sig_pairs <- significant_pairs

  if (nrow(sig_pairs) > 0) {

    comps <- strsplit(sig_pairs$Comparison, "-")

    sig_pairs$group1 <- sapply(comps, function(x) trimws(x[1]))
    sig_pairs$group2 <- sapply(comps, function(x) trimws(x[2]))
  }

  group_levels <- levels(data_long$group)

  sig_pairs$x1 <- match(sig_pairs$group1, group_levels)
  sig_pairs$x2 <- match(sig_pairs$group2, group_levels)

  sig_pairs$signif <- ifelse(sig_pairs$p_adj < 0.001, "***",
                             ifelse(sig_pairs$p_adj < 0.01, "**",
                                    ifelse(sig_pairs$p_adj < 0.05, "*", "")))

  y_max <- max(data_long$value, na.rm = TRUE)
  y_range <- diff(range(data_long$value, na.rm = TRUE))

  step <- 0.08 * y_range

  sig_pairs$y <- y_max + seq_len(nrow(sig_pairs)) * step

  # --------------------------
  # Subtitle
  # --------------------------
  subtitle_text <- .make_subtitle_anova(
    omega_sq = omega_sq,
    p_value = p_anova
  )

  # --------------------------
  # Colors
  # --------------------------
  # Vivid colors
  vivid_colors <- scales::hue_pal()(length(unique(data_long$group)))

  # mono
  n <- length(groups)

  mono_colors <- gray.colors(
    n,
    start = 0.9,
    end = 0.1
  )

  # --------------------------
  # Boxplot
  # --------------------------
  if (style == "boxplot") {
    g <- ggplot2::ggplot(data_long, ggplot2::aes(x = group, y = value, fill = group)) +
      ggplot2::geom_boxplot(alpha = 0.7, outlier.shape = NA, width = 0.7, linewidth = 0.7) +
      ggplot2::geom_jitter(width = 0.1, alpha = 0.2, color = "grey25") +
      ggplot2::labs(title = title, subtitle = subtitle_text, x = "", y = ylab) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::theme(legend.position = "none",
                     axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 12))

}

  # --------------------------
  # Violin plot
  # --------------------------
  if (style == "violin") {
    g <- ggplot2::ggplot(data_long, ggplot2::aes(x = group, y = value, fill = group)) +
      ggplot2::geom_violin(trim = FALSE, alpha = 0.6, color = NA, adjust = 0.6) +
      ggplot2::geom_boxplot(width = 0.18, outlier.shape = NA,
                            color = "gray20", linewidth = 0.4) +
      ggplot2::geom_point(position = ggplot2::position_jitter(width = 0.1),
                          alpha = 0.2, size = 1.8, color = "gray25") +
      ggplot2::labs(title = title, subtitle = subtitle_text, x = "", y = ylab) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(legend.position = "none",
                     axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 12))
}

  # --------------------------
  # monochrome premium
  # --------------------------
  if (style == "mono") {
    g <- ggplot2::ggplot(data_long, ggplot2::aes(x = group, y = value, fill = group)) +
      ggplot2::geom_boxplot(alpha = 0.7, outlier.shape = NA, width = 0.7, linewidth = 0.7, color = "black") +
      ggplot2::geom_jitter(width = 0.1, alpha = 0.2, color = "grey25") +
      ggplot2::labs(title = title, subtitle = subtitle_text, x = "", y = ylab) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::scale_fill_manual(values = mono_colors) +
      ggplot2::theme(legend.position = "none",
                     axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 12))
}

  # --------------------------
  # Half eye
  # --------------------------
  if (style == "halfeye") {
    if (!requireNamespace("ggdist", quietly = TRUE)) {
      stop("For 'halfeye' style, please install the 'ggdist' package")
    }
    g <- ggplot2::ggplot(data_long, ggplot2::aes(x = group, y = value, fill = group)) +
      ggdist::stat_halfeye(alpha = .6, trim = FALSE, adjust = 0.6, width = 0.6,
                           .width = c(0.5, 0.8, 0.95),
                           justification = -0.2,
                           slab_color = "gray20",
                           interval_color = "gray20") +
      ggplot2::geom_point(position = ggplot2::position_nudge(x = 0.15),
                          size = 1.1, alpha = 0.2, color = "grey25") +
      ggdist::stat_pointinterval(position = ggplot2::position_nudge(x = 0.2),
                                 point_color = "black",
                                 interval_color = "black",
                                 .width = 0.95) +
      ggplot2::labs(title = title, subtitle = subtitle_text, x = "", y = ylab) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(legend.position = "none",
                     axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 12))
}

  # --------------------------
  # Annotation and print
  # --------------------------
  g <- g + .add_significance(sig_pairs, y_range)

  print(g)

  # --------------------------
  # Output
  # --------------------------
  obj <- (list(
    type = "ANOVA",
    p_anova = p_anova,
    omega_sq = omega_sq,
    omega_ci = c(omega_ci_low, omega_ci_high),
    normal = normal,
    p_normal = p_normal,
    homogeneous = homogeneous,
    p_levene = p_levene,
    means_sd = means_sd,
    tukey = tukey_df,
    significant_pairs = significant_pairs,
    model = model
    )
  )

  # --------------------------
  # Return
  # --------------------------
  if (verbose) {
    .print_header("One-way ANOVA")

    .print_block("Statistics", function() {

      cat("F statistic = ",
          round(anova_tab[1, "F value"], 3),
          " | df = ",
          anova_tab[1, "Df"], ", ",
          anova_tab[nrow(anova_tab), "Df"],
          " | p = ",
          .format_pval(p_anova),
          "\n", sep = "")

      cat("Omega^2 = ",
          round(omega_sq, 3),
          " [",
          round(omega_ci_low, 3), ", ",
          round(omega_ci_high, 3),
          "]\n", sep = "")

    })

    # --- Post-hoc output ---

    .print_header("Post-hoc: Tukey HSD")

    .print_block("Significant comparisons", function() {

      if (nrow(significant_pairs) == 0) {

        cat("No significant comparisons (p < 0.05)\n")

      } else {

        for (i in seq_len(nrow(significant_pairs))) {

          r <- significant_pairs[i, ]

          comps <- unlist(strsplit(r$Comparison, "-"))

          g1 <- trimws(comps[1])
          g2 <- trimws(comps[2])

          cat(g1, " vs ", g2, "\n", sep = "")

          cat(
            "Mean difference = ",
            round(r$diff, 3),
            " [",
            round(r$lwr, 3), ", ",
            round(r$upr, 3),
            "]\n",
            sep = ""
          )

          cat(
            "p (adj) = ",
            .format_pval(r$p_adj),
            "\n\n",
            sep = ""
          )
        }
      }
    })
  }

  return(invisible(list(result = obj)))
}

# -------------------------------
# Imports
# -------------------------------

# stats
# ---------------------------------------------------------------------------------
#' @importFrom stats sd aggregate shapiro.test ks.test t.test wilcox.test var.test
#' @importFrom stats TukeyHSD complete.cases setNames na.omit
#' @importFrom stats chisq.test confint formula kruskal.test lm nobs p.adjust qnorm quantile reformulate dist

# grDevices
# ---------------------------------------------------------------------------------
#' @importFrom grDevices gray.colors

# utils
# ---------------------------------------------------------------------------------
#' @importFrom utils globalVariables combn

# dplyr / purrr / tidyverse helpers
# ---------------------------------------------------------------------------------
#' @importFrom dplyr %>%
#' @importFrom purrr map_dfr
#' @importFrom tibble tibble as_tibble
#' @importFrom tidyr pivot_longer
#' @importFrom scales hue_pal

# Specific packages for analysis and visualization
# ---------------------------------------------------------------------------------
#' @importFrom car leveneTest
#' @importFrom ggplot2 ggplot aes geom_boxplot geom_jitter geom_violin geom_point geom_segment annotate labs theme_minimal scale_fill_manual element_text
#' @importFrom RColorBrewer brewer.pal
#' @importFrom ggExtra ggMarginal

NULL

# ============================
# Auxiliary print functions
# ============================

#' @keywords internal
.print_header <- function(title) {
  cat("\n")
  cat(strrep("=", 50), "\n")
  cat(title, "\n")
  cat(strrep("=", 50), "\n")
}

#' @keywords internal
.print_block <- function(title, content, width = 40) {
  cat("\n", title, "\n", sep = "")
  cat(strrep("-", width), "\n")
  content()
  cat(strrep("-", width), "\n")
}

#' Test for interaction between a continuous predictor and a grouping variable
#'
#' Fits a linear model of the form \code{y ~ x * by} to evaluate whether the
#' association between a continuous predictor and an outcome differs across
#' groups. Optionally produces a publication-ready visualization of
#' group-specific regression lines.
#'
#' @param x Numeric vector representing the continuous predictor.
#' @param y Numeric vector representing the continuous outcome.
#' @param by Grouping variable defining the interaction. Must be coercible
#'   to a factor with at least two levels.
#' @param title Optional title title for the plot.
#' @param xlab Optional x-axis label.
#' @param ylab Optional y-axis label.
#' @param plot Logical. Should a plot be generated?
#' @param style Plot style. One of \code{"clean"}, \code{"CI"} or \code{"facet"}.
#' @param conf.level Confidence level for the interaction interval
#'   (default: 0.95).
#' @param help Logical. If TRUE, shows a detailed explanation of the function.
#'        Default: FALSE.
#' @param verbose Logical. If TRUE, prints detailed messages. Default: TRUE.
#'
#' @return An object of class \code{"test.interaction"} containing:
#' \itemize{
#'   \item \code{model}: the fitted linear model,
#'   \item \code{interaction}: estimated interaction effects with confidence intervals,
#'   \item \code{plot}: a \code{ggplot} object (if \code{plot = TRUE}).
#' }
#'
#' @details
#' The interaction coefficient (\eqn{\beta}) represents the difference in
#' regression slopes between groups, conditional on the reference level
#' of \code{by}. The sign and magnitude of this coefficient depend on the
#' chosen reference group.
#'
#' Confidence intervals are emphasized as the primary inferential quantity.
#'
#' @examples
#'
#' # Simple example: different trends between groups
#'
#' set.seed(123)
#'
#' n <- 60
#'
#' marker <- rnorm(n, 10, 2)
#' group  <- rep(c("Control", "Treatment"), each = n/2)
#'
#' # Same intercept, different slopes
#' response <- 2 +
#'   ifelse(group == "Control", 0.5, 1.2) * marker +
#'   rnorm(n, 0, 1)
#'
#' test.interaction(marker, response, group)
#'
#' @export

test.interaction <- function(
    x,
    y,
    by,
    title = NULL,
    xlab = NULL,
    ylab = NULL,
    plot = TRUE,
    style = c("clean", "CI", "facet"),
    conf.level = 0.95,
    verbose = TRUE,
    help = FALSE
) {

  style <- match.arg(style)

  if (help) {
    if (verbose) {
      message("
Function test.interaction()

Description:
  Tests whether the association between a continuous predictor and an outcome
  differs across groups using a linear model (y ~ x * by).

Interpretation:
  The interaction term represents the difference in slopes between groups.

Example:

n <- 60
df_bio <- data.frame(
marker   = rnorm(n, mean = 10, sd = 2),
group    = rep(c('Control', 'Treatment'), each = n / 2)
)

df_bio$response <- with(
df_bio,
ifelse(
group == 'Control',
2 + 0.5 * marker + rnorm(n, 0, 1),
2 + 1.2 * marker + rnorm(n, 0, 1)
)
)

test.interaction(
x  = df_bio$marker,
y  = df_bio$response,
by = df_bio$group)

")
    }
    return(invisible(NULL))
  }

  pretty_interaction_term <- function(term, by_var) {
    # Remove 'x:' prefix
    term <- sub("^x:", "", term)

    # Remove variable name (by)
    term <- sub(paste0("^", by_var), "", term)

    paste0("Interaction (", term, ")")
  }

  # ---------------------------
  # Input validation
  # ---------------------------
  if (!is.numeric(x) || !is.numeric(y)) {
    stop("x and y must be numeric vectors.", call. = FALSE)
  }

  if (length(x) != length(y) || length(y) != length(by)) {
    stop("x, y and by must have the same length.", call. = FALSE)
  }

  df <- data.frame(
    x  = x,
    y  = y,
    by = factor(by)
  )

  df <- df[complete.cases(df), ]

  if (nlevels(df$by) < 2) {
    stop("'by' must have at least two levels.", call. = FALSE)
  }

  # Variable labels (for plots)
  x_var  <- deparse(substitute(x))
  y_var  <- deparse(substitute(y))
  by_var <- deparse(substitute(by))

  clean_label <- function(label) {
    label <- gsub(".*\\$", "", label)
    label <- gsub("_", " ", label)
    tools::toTitleCase(label)
  }

  x_var  <- clean_label(x_var)
  y_var  <- clean_label(y_var)
  by_var <- clean_label(by_var)

  # ---------------------------
  # Model
  # ---------------------------
  fit <- lm(y ~ x * by, data = df)

  coef_tbl <- summary(fit)$coefficients
  ci_tbl   <- confint(fit, level = conf.level)

  interaction_terms <- grep("^x:|:x$", rownames(coef_tbl), value = TRUE)

  interaction_info <- lapply(interaction_terms, function(term) {

    term_pretty <- pretty_interaction_term(term, by_var)

    list(
      term = term_pretty,
      raw_term = term,
      beta = coef_tbl[term, "Estimate"],
      p    = coef_tbl[term, "Pr(>|t|)"],
      ci   = ci_tbl[term, ]
    )
  })

  summary_table <- aggregate(
    y ~ by,
    data = df,
    function(z) c(
      Mean = mean(z, na.rm = TRUE),
      SD   = sd(z, na.rm = TRUE),
      N    = sum(!is.na(z))
    )
  )

  summary_table <- do.call(data.frame, summary_table)

  main_interaction <- interaction_info[[1]]

  # ---------------------------
  # Plot
  # ---------------------------
  g <- NULL

  if (plot) {

    base_plot <- ggplot2::ggplot(
      df,
      ggplot2::aes(x = x, y = y, color = by, fill = by)
    ) +
      ggplot2::theme_minimal(base_size = 12)

    if (style == "clean") {
      g <- base_plot +
        ggplot2::geom_point(alpha = 0.6, size = 2) +
        ggplot2::geom_smooth(method = "lm", se = FALSE)

    } else if (style == "CI") {
      g <- base_plot +
        ggplot2::geom_point(alpha = 0.5, size = 1.8) +
        ggplot2::geom_smooth(
          method = "lm",
          se = TRUE,
          level = conf.level,
          alpha = 0.25
        )

    } else if (style == "facet") {
      g <- ggplot2::ggplot(df, ggplot2::aes(x, y)) +
        ggplot2::geom_point(alpha = 0.6, size = 2) +
        ggplot2::geom_smooth(method = "lm", se = FALSE) +
        ggplot2::facet_wrap(~ by) +
        ggplot2::theme_minimal(base_size = 12)
    }

    if (style %in% c("clean", "CI")) {
      g <- g +
        ggplot2::labs(
          title = if (is.null(title)) paste(y_var, "vs", x_var) else title,
          subtitle = .build_subtitle_interaction(
            main_interaction$beta,
            main_interaction$p
          ),
          x = if (is.null(xlab)) x_var else xlab,
          y = if (is.null(ylab)) y_var else ylab,
          color = by_var,
          fill  = by_var
        )
    } else {
      g <- g +
        ggplot2::labs(
          title = if (is.null(title)) paste(y_var, "vs", x_var) else title,
          subtitle = .build_subtitle_interaction(
            main_interaction$beta,
            main_interaction$p
          ),
          x = if (is.null(xlab)) x_var else xlab,
          y = if (is.null(ylab)) y_var else ylab
        )
    }
  }

  # ---------------------------
  # Return
  # ---------------------------
  out <- list(
    call = match.call(),
    model = fit,
    interaction = interaction_info,
    summary = summary_table,
    conf.level = conf.level,
    plot = g,
    method = "Linear model interaction",
    verbose = verbose
  )

  class(out) <- "test.interaction"
  out
}


# ---------------------------
# S3 Output
# ---------------------------
#' @method print test.interaction
#' @param ... Additional arguments passed to other print methods (currently ignored)
#' @export
#' @rdname test.interaction

print.test.interaction <- function(x, ...) {

  if (!isTRUE(x$verbose)) {
    return(invisible(x))
  }

  .print_header(x$method)

  .print_block("Model", function() {

    cat("Formula:", deparse(formula(x$model)), "\n")
    cat("Observations:", nobs(x$model), "\n")

  })

  .print_block("Summary by group", function() {
    print(x$summary, row.names = FALSE)
  })

  .print_block("Statistics", function() {

    for (info in x$interaction) {

      cat(
        info$term, "\n",
        "  Beta = ", round(info$beta, 3),
        " | CI = [",
        round(info$ci[1], 3), ", ",
        round(info$ci[2], 3),
        "]",
        " | p = ",
        .format_pval(info$p),
        "\n\n",
        sep = ""
      )

    }
  })

  if (!is.null(x$plot)) {
    print(x$plot)
  }

  invisible(x)
}

#' pre.test() — Automatic statistical test suggestion
#'
#' Automatically identifies whether the input data are numeric or categorical
#' and suggests the most appropriate statistical test.
#'
#' @param ... Two or more vectors (numeric or categorical), or a data.frame with >= 2 columns
#' @param alpha Significance level. Default = 0.05
#' @param help Logical. If TRUE, shows detailed help
#' @param verbose Logical. If TRUE, prints informative messages
#' @importFrom tidyselect everything
#' @return Invisible list with normality results, homogeneity or contingency table,
#' and test recommendation
#' @export

pre.test <- function(..., alpha = 0.05, help = FALSE, verbose = TRUE) {

  # ============================
  # Capture input
  # ============================
  args <- list(...)

  # Data frame input (>= 2 columns)
  if (length(args) == 1 && is.data.frame(args[[1]]) && ncol(args[[1]]) >= 2) {
    groups <- lapply(args[[1]], identity)
    group_names <- colnames(args[[1]])
  } else {
    groups <- args
    raw_names <- as.character(match.call(expand.dots = FALSE)$...)
    group_names <- sub("^.*\\$", "", raw_names)
  }

  # ============================
  # Help message
  # ============================
  if (help || length(groups) < 2) {

    if (verbose) {
      message("
pre.test()

Description:
  Automatically identifies whether the data are numeric or categorical
  and suggests the most appropriate statistical test.

Usage:
  - Accepts vectors or a data frame with 2 or more columns.

Example (data frame):
  df <- data.frame(
    control   = c(1, 2, 3, 4, 5),
    treatment = c(2, 3, 4, 5, 6)
  )
  pre.test(df)
")
    }

    return(invisible(NULL))
  }

  # ============================
  # Detect data type
  # ============================
  are_numeric <- all(vapply(groups, is.numeric, logical(1)))
  are_categorical <- all(
    vapply(groups, function(x) is.factor(x) || is.character(x), logical(1))
  )

  if (!are_numeric && !are_categorical) {
    stop("All inputs must be either numeric or categorical.")
  }

  # =====================================================================
  # NUMERIC DATA
  # =====================================================================
  if (are_numeric) {

    values <- unlist(groups)
    group_factor <- factor(
      rep(group_names, times = vapply(groups, length, integer(1)))
    )

    data_long <- data.frame(
      value = values,
      group = group_factor
    )

    # ----------------------------
    # Normality test selection
    # ----------------------------
    normality_test <- function(x) {

      n <- length(x)

      if (n <= 50) {
        p_value <- stats::shapiro.test(x)$p.value
        method <- "Shapiro-Wilk"
      } else if (n <= 300) {
        p_value <- nortest::ad.test(x)$p.value
        method <- "Anderson-Darling"
      } else {
        ks_res <- stats::ks.test(
          x,
          "pnorm",
          mean = mean(x),
          sd = stats::sd(x)
        )
        p_value <- ks_res$p.value
        method <- "Kolmogorov-Smirnov"
      }

      list(p_value = p_value, method = method)
    }

    normality_results <- lapply(groups, normality_test)

    p_values <- vapply(normality_results, function(x) x$p_value, numeric(1))
    methods <- vapply(normality_results, function(x) x$method, character(1))

    normality_df <- data.frame(
      group = group_names,
      n = vapply(groups, length, integer(1)),
      method = methods,
      p_value = signif(p_values, 3),
      normal = p_values > alpha
    )

    # ----------------------------
    # Homogeneity of variance
    # ----------------------------
    if (length(groups) > 2) {
      p_homogeneity <- car::leveneTest(value ~ group, data = data_long)$`Pr(>F)`[1]
    } else {
      p_homogeneity <- stats::var.test(groups[[1]], groups[[2]])$p.value
    }

    homogeneous <- p_homogeneity > alpha

    # ----------------------------
    # Test recommendation
    # ----------------------------
    if (length(groups) == 2) {

      if (all(normality_df$normal)) {
        recommendation <- if (homogeneous) {
          "t-test"
        } else {
          "Welch's t-test"
        }
      } else {
        recommendation <- "Mann-Whitney test"
      }

    } else {

      if (all(normality_df$normal) && homogeneous) {
        recommendation <- "ANOVA with Tukey post hoc test"
      } else {
        recommendation <- "Kruskal-Wallis with Dunn post hoc test"
      }
    }

    # ----------------------------
    # Verbose output
    # ----------------------------
    if (verbose) {

      separator <- paste(rep("=", 50), collapse = "")

      message("Normality test results:")
      message(separator)
      print(normality_df)
      message(separator)

      message(
        sprintf(
          "Homogeneity of variances: %s (p = %.3f)",
          ifelse(homogeneous, "homogeneous", "heterogeneous"),
          p_homogeneity
        )
      )

      message(separator)
      message(sprintf("Test recommendation: %s", recommendation))
      message(separator)
    }

    # ----------------------------
    # Return
    # ----------------------------
    return(invisible(list(
      normality = normality_df,
      p_homogeneity = p_homogeneity,
      homogeneous = homogeneous,
      recommendation = recommendation
    )))
  }

  # =====================================================================
  # CATEGORICAL DATA
  # =====================================================================
  if (are_categorical) {

    if (length(args) == 1 && is.data.frame(args[[1]]) && ncol(args[[1]]) == 2) {

      df <- args[[1]]

      df_long <- tidyr::pivot_longer(
        df,
        cols = tidyselect::everything(),
        names_to = "group",
        values_to = "category"
      )

      contingency_table <- table(df_long$group, df_long$category)

    } else {

      categories <- lapply(groups, as.factor)
      contingency_table <- table(categories[[1]], categories[[2]])
    }

    recommendation <- if (any(contingency_table < 5)) {
      "Fisher's exact test"
    } else {
      "Chi-squared test"
    }

    if (verbose) {
      message("Contingency table:")
      print(contingency_table)
      message("Association test recommendation: ", recommendation)
    }

    # ----------------------------
    # Return
    # ----------------------------
    return(invisible(list(
      contingency_table = contingency_table,
      recommendation = recommendation
    )))
  }
}

#' Paired t-test with advanced visualizations
#'
#' Performs a paired t-test between two numeric vectors (e.g., before vs after)
#' or between two numeric columns of a data frame.
#' Includes four visualization styles (boxplot, violin, mono, and half-eye).
#'
#' @param ... Two numeric vectors of equal length, or
#'   a data frame with exactly two numeric columns.
#' @param title Plot title.
#' @param xlab X-axis label.
#' @param ylab Y-axis label.
#' @param style Plot style:
#'   \itemize{
#'     \item \code{1} Premium boxplot
#'     \item \code{2} Violin + minimal boxplot
#'     \item \code{3} mono
#'     \item \code{4} Half-eye (ggdist)
#'   }
#' @param connect Logical. If TRUE, connects paired observations.
#' @param help If TRUE, displays detailed help.
#' @param verbose If TRUE, prints progress messages.
#'
#' @return An invisible list containing:
#' \describe{
#'   \item{summary_table}{Group means and standard deviations}
#'   \item{test_result}{t-test result object (stats::t.test)}
#'   \item{data}{Data frame used for plotting}
#'   \item{plot}{ggplot2 object}
#' }
#'
#' @export
#'
#' @examples
#' before <- c(13, 12, 15, 14)
#' after  <- c(9, 11, 10, 10)
#' test.tpaired(before, after)
#'

test.tpaired <- function(
    ...,
    title = "Paired t-test",
    xlab = "",
    ylab = "Value",
    style = c("boxplot", "violin", "mono", "halfeye"),
    connect = TRUE,
    help = FALSE,
    verbose = TRUE
) {

  args <- list(...)
  style <- match.arg(style)

  # ------------------------------
  # Help
  # ------------------------------
  if (help) {
    message(
      "Function test.tpaired()

Accepted input:
 - Two numeric vectors of equal length
 - Or a data frame with exactly two numeric columns

# Example
before <- c(13, 12, 15, 14)
after  <- c(9, 11, 10, 10)
test.tpaired(before, after)

Returns:
 A list with summary_table, t.test result, plotting data and plot"
    )
    return(invisible(NULL))
  }

  # ------------------------------
  # Flexible input
  # ------------------------------
  if (length(args) == 1 && is.data.frame(args[[1]]) && ncol(args[[1]]) == 2) {
    df <- args[[1]]
    if (!all(sapply(df, is.numeric)))
      stop("The data frame must contain exactly two numeric columns.")
    x <- df[[1]]
    y <- df[[2]]
    names <- colnames(df)
  } else {
    if (length(args) != 2)
      stop("Provide two numeric vectors or a data frame with two columns.")

    x <- args[[1]]
    y <- args[[2]]

    if (!is.numeric(x) || !is.numeric(y))
      stop("Both vectors must be numeric.")

    if (length(x) != length(y))
      stop("Vectors must have the same length (paired test).")

    names <- as.character(match.call(expand.dots = FALSE)$...)[1:2]
  }

  # Remove missing pairs
  ok <- complete.cases(x, y)
  x <- x[ok]
  y <- y[ok]

  if (length(x) < 3) stop("At least 3 valid paired observations are required.")

  # ------------------------------
  # Statistical test_result
  # ------------------------------
  test_result <- stats::t.test(x, y, paired = TRUE)

  # Differences
  d <- x - y

  # Effect size: Cohen's dz
  dz <- mean(d) / sd(d)

  # Bootstrap CI
  res_boot <- .boot_one_sample(
    d,
    stat_fun = function(z)
      mean(z) / sd(z)
  )

  ci_low  <- res_boot$ci_low
  ci_high <- res_boot$ci_high

  # Labels
  p_value <- test_result$p.value
  p_numeric <- signif(p_value, 3)
  p_label   <- .format_pval(p_value)

  summary_table <- data.frame(
    Group = names,
    Mean  = round(c(mean(x), mean(y)), 2),
    SD    = round(c(sd(x), sd(y)), 2)
  )

  attr(summary_table, "effect_size") <- list(
    dz = dz,
    CI = c(
      low = ci_low,
      high = ci_high
    )
  )

  # ------------------------------
  # Final data frame
  # ------------------------------
  data <- data.frame(
    id = seq_along(x),
    group = factor(
      rep(names, each = length(x)),
      levels = names
    ),
    value = c(x, y)
  )

  # ============================
  # Labels and colors
  # ============================
  signif_label <- if (p_value < 0.001) {
    "***"
  } else if (p_value < 0.01) {
    "**"
  } else if (p_value < 0.05) {
    "*"
  } else {
    ""
  }

  mean_diff <- mean(d)

  y_pos <- max(data$value, na.rm = TRUE) +
    0.1 * diff(range(data$value, na.rm = TRUE))

  # --- Optional paired-line layer ---
  lines <- function() {
    if (!connect) return(NULL)
    ggplot2::geom_line(
      data = data,
      ggplot2::aes(x = group, y = value, group = id),
      color = "gray40",
      linewidth = 0.5,
      alpha = 0.4
    )
  }

  # --- Colors ---
  vivid_colours <- scales::hue_pal()(length(unique(data$group)))
  mono_colors <- c("grey75", "grey25")

  # ============================
  # STYLE 1 (Boxplot + jitter)
  # ============================
  if (style == "boxplot") {
    g <- ggplot2::ggplot(data, ggplot2::aes(group, value, fill = group)) +
      ggplot2::geom_boxplot(alpha = .7, outlier.shape = NA, width = 0.7, linewidth = 0.7) +
      lines() +
      ggplot2::geom_point(
        position = ggplot2::position_jitter(width = .1),
        alpha = .2,
        size = 1.8,
        color = "grey25"
      ) +
      ggplot2::annotate(
        "text", x = mean(1:2), y = y_pos,
        label = signif_label, size = 6,
        col = "grey25"
      ) +
      ggplot2::scale_fill_manual(values = vivid_colours) +
      ggplot2::scale_y_continuous(
        expand = ggplot2::expansion(mult = c(0.05, 0.15))
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::labs(
        title = title,
        subtitle = .build_subtitle(mean_diff, p_value, "mean diff"),
        x = xlab,
        y = ylab
      ) +
      ggplot2::theme(
        legend.position = "none",
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 12)
      )
  }

  # ============================
  # STYLE 2 (Violin)
  # ============================
  if (style == "violin") {
    g <- ggplot2::ggplot(data, ggplot2::aes(group, value, fill = group)) +
      ggplot2::geom_violin(trim = FALSE, alpha = .6, color = NA, adjust = .6) +
      ggplot2::geom_boxplot(width = .18, outlier.shape = NA, color = "gray20") +
      lines() +
      ggplot2::geom_point(
        position = ggplot2::position_jitter(width = .1),
        alpha = .2, size = 1.8, color = "gray25"
      ) +
      ggplot2::annotate(
        "text", x = mean(1:2), y = y_pos,
        label = signif_label, size = 6,
        col = "grey25"
      ) +
      ggplot2::scale_fill_manual(values = vivid_colours) +
      ggplot2::scale_y_continuous(
        expand = ggplot2::expansion(mult = c(0.05, 0.15))
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::labs(
        title = title,
        subtitle = .build_subtitle(mean_diff, p_value, "mean diff"),
        x = xlab,
        y = ylab
      ) +
      ggplot2::theme(
        legend.position = "none",
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 12)
      )
  }

  # ============================
  # STYLE 3 (monochrome premium)
  # ============================
  if (style == "mono") {
    g <- ggplot2::ggplot(data, ggplot2::aes(x = group, y = value, fill = group)) +
      ggplot2::geom_boxplot(alpha = 0.75, outlier.shape = NA, width = 0.7, linewidth = 0.7) +
      lines() +
      ggplot2::geom_jitter(width = 0.1, alpha = 0.2, color = "grey25", size = 1.8) +
      ggplot2::annotate(
        "text", x = mean(1:2), y = y_pos,
        label = signif_label, size = 6,
        col = "grey25"
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::scale_fill_manual(values = mono_colors) +
      ggplot2::labs(
        title = title,
        subtitle = .build_subtitle(mean_diff, p_value, "mean diff"),
        x = "",
        y = ylab
      ) +
      ggplot2::theme(
        legend.position = "none",
        plot.margin = ggplot2::margin(5.5, 5.5, 10, 5.5),
        axis.text.x = ggplot2::element_text(
          angle = 45, hjust = 1, size = 12
        )
      )
  }

  # ============================
  # STYLE 4 (ggdist half-eye + median)
  # ============================
  if (style == "halfeye") {
    if (!requireNamespace("ggdist", quietly = TRUE))
      stop("Package 'ggdist' is required for style = halfeye'.")

    g <- ggplot2::ggplot(data, ggplot2::aes(group, value, fill = group)) +
      ggdist::stat_halfeye(alpha = .6, adjust = 0.6, trim = FALSE, .width = 0.95) +
      lines() +
      ggplot2::geom_point(
        position = ggplot2::position_jitter(width = .1),
        alpha = .4
      ) +
      ggplot2::annotate(
        "text", x = mean(1:2), y = y_pos,
        label = signif_label, size = 6,
        col = "grey25"
      ) +
      ggplot2::scale_y_continuous(
        expand = ggplot2::expansion(mult = c(0.05, 0.15))
      ) +
      ggplot2::scale_fill_manual(values = vivid_colours) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::labs(
        title = title,
        subtitle = .build_subtitle(mean_diff, p_value, "mean diff"),
        x = xlab,
        y = ylab
      ) +
      ggplot2::theme(
        legend.position = "none",
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 12)
      )
  }

  print(g)

  # ------------------------------
  # Output
  # ------------------------------
  obj <- list(
    summary_table = summary_table,
    test = test_result,
    data = data,
    plot = g
  )

  # ------------------------------
  # Return
  # ------------------------------
  if (verbose) {

    .print_header("Paired t-test")

    .print_block("Summary", function() {
      print(summary_table, row.names = FALSE)
    })

    .print_block("Statistics", function() {

      cat(
        "t statistic = ", round(test_result$statistic, 3),
        " | df = ", round(test_result$parameter, 1),
        " | p = ", p_label,
        "\n",
        sep = ""
      )

      cat(
        "Effect size (Cohen's dz) = ",
        round(dz, 2),
        " [",
        round(ci_low, 2), ", ",
        round(ci_high, 2),
        "]\n",
        sep = ""
      )
    })
  }

  return(invisible(list(result = obj)))
}

#' Fisher's Exact Test
#'
#' Performs Fisher's Exact Test using two categorical vectors or a data frame with two columns,
#' constructing a contingency table and optionally generating graphical visualizations.
#'
#' @param x Categorical vector or data frame with two columns.
#' @param y Categorical vector (required if x_var is a vector).
#' @param title Plot title (string). Default: "Fisher's Exact Test"
#' @param xlab Name of the x-axis in the plot (string). Default: NULL (uses variable name)
#' @param ylab Name of the y-axis in the plot (string). Default: "Proportion"
#' @param style Plot style. Controls the visualization type
#' @param show_table Logical. If TRUE, prints the contingency table to the console. Default: TRUE
#' @param help Logical. If TRUE, shows detailed function explanation. Default: FALSE
#' @param verbose Logical. If TRUE, prints detailed test messages. Default: TRUE
#'
#' @return Invisible object containing the Fisher test result.
#' @export
#'
#' @examples
#' data <- data.frame(control = c("healthy","healthy","sick","sick","sick"),
#'                    treatment = c("healthy","healthy","healthy","healthy","sick"))
#' test.fisher(data)

test.fisher <- function(x, y = NULL,
                        title = "Fisher's Exact Test",
                        xlab = NULL,
                        ylab = "Proportion",
                        style = c("stacked", "barplot", "mosaic", "pie"),
                        show_table = TRUE,
                        help = FALSE,
                        verbose = TRUE) {

  style <- match.arg(style)

  # Help
  if (help || missing(x)) {
    if (verbose) {
      message(
        "Function test.fisher()

Description:
  Performs Fisher's Exact Test to evaluate association between
  two categorical variables.

When to use:
  - Small samples
  - Expected frequencies < 5
  - Especially for 2x2 tables

Effect size:
  - Odds Ratio (OR)
  - log(OR) with confidence interval

Example:
data <- data.frame(control = c('healthy','healthy','sick','sick','sick'),
                   treatment = c('healthy','healthy','healthy','healthy','sick'))

test.fisher(data)

"
      )
    }
    return(invisible(NULL))
  }

  # Required packages
  required_packages <- c("ggplot2", "dplyr", "tidyr", "vcd")
  for (pkg in required_packages) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
      stop(
        paste0(
          "Package '", pkg,
          "' is not installed. Install it first."
        ),
        call. = FALSE
      )
    }
  }

  # -----------------------------
  # Input handling
  # -----------------------------

  if (is.data.frame(x)) {

    if (ncol(x) != 2) {
      stop("The data frame must contain exactly two columns.", call. = FALSE)
    }

    column_names <- colnames(x)

    data_long <- tidyr::pivot_longer(
      x,
      cols = tidyselect::everything(),
      names_to = "group",
      values_to = "category"
    )

    data_long$group <- factor(
      data_long$group,
      levels = column_names
    )

    group <- data_long$group
    category <- data_long$category

    name_x <- column_names[1]
    name_y <- column_names[2]

  } else {

    if (is.null(y)) {
      stop("Argument 'y' must be provided if 'x' is a vector.", call. = FALSE)
    }

    if (length(x) != length(y)) {
      stop("Both variables must have the same length.", call. = FALSE)
    }

    group <- x
    category <- y

    name_x <- deparse(substitute(x))
    name_y <- deparse(substitute(y))

    name_x <- sub(".*\\$", "", name_x)
    name_y <- sub(".*\\$", "", name_y)
  }

  if (is.null(xlab)) xlab <- name_x
  if (is.null(ylab)) ylab <- "Proportion"


  # -----------------------------
  # Contingency table
  # -----------------------------

  contingency_table <- table(group, category)

  if (verbose && show_table) {
    message("Observed contingency table:")
    print(contingency_table)
  }

  # -----------------------------
  # Fisher test
  # -----------------------------

  test <- stats::fisher.test(contingency_table)

  # -----------------------------
  # Effect size (Odds Ratio)
  # -----------------------------

  or <- as.numeric(test$estimate)
  ci <- test$conf.int

  log_or <- log(or)
  log_ci <- log(ci)

  # -----------------------------
  # Prepare plot data
  # -----------------------------

  df_plot <- data.frame(group = group, category = category)

  df_prop <- df_plot |>
    dplyr::group_by(group, category) |>
    dplyr::summarise(n = dplyr::n(), .groups = "drop") |>
    dplyr::group_by(group) |>
    dplyr::mutate(prop = n / sum(n))

  # --- Subtitle ---
  subtitle_text <- .make_subtitle_fisher(
    or = or,
    p_value = test$p.value
  )

  # --- Colors ---
  vivid_colors <- scales::hue_pal()(length(unique(df_prop$group)))

  # -----------------------------
  # Plots
  # -----------------------------

  # --- Stacked ---
  if (style == "stacked") {

    g <- ggplot2::ggplot(
      df_prop,
      ggplot2::aes(x = group, y = prop, fill = category)
    ) +
      ggplot2::geom_bar(stat = "identity") +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::labs(
        title = title,
        subtitle = subtitle_text,
        y = ylab,
        fill = name_y
      ) +
      ggplot2::theme_minimal()

  }

  # --- Barplot ---
  if (style == "barplot") {

    g <- ggplot2::ggplot(
      df_prop,
      ggplot2::aes(x = group, y = prop, fill = category)
    ) +
      ggplot2::geom_bar(
        stat = "identity",
        position = ggplot2::position_dodge()
      ) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::labs(
        title = title,
        subtitle = subtitle_text,
        y = ylab,
        fill = name_y
      ) +
      ggplot2::theme_minimal()
  }

  # --- Mosaic ---
  if (style == "mosaic") {

    vcd::mosaic(
      contingency_table,
      shade = TRUE,
      legend = TRUE,
      main = paste0(title, "\n", subtitle_text)
    )
  }

  # --- Pie ---
  # --- Pie ---
  if (style == "pie") {

    g <- ggplot2::ggplot(
      df_prop,
      ggplot2::aes(x = "", y = prop, fill = category)
    ) +
      ggplot2::geom_bar(stat = "identity", width = 1) +
      ggplot2::coord_polar("y") +
      ggplot2::facet_wrap(~ group) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::theme_void(base_size = 12) +
      ggplot2::labs(
        title = title,
        subtitle = subtitle_text,
        fill = name_y
      ) +
      ggplot2::theme(
        plot.title = ggplot2::element_text(
          hjust = 0.5,
          size = 14
        ),
        plot.subtitle = ggplot2::element_text(
          hjust = 0.5,
          size = 11,
          margin = ggplot2::margin(b = 10)
        ),
        strip.text = ggplot2::element_text(
          size = 12
        )
      )
  }

  if (style != "mosaic") print(g)

  # -----------------------------
  # Output
  # -----------------------------
  obj <- list(
    type = "Fisher",
    p = test$p.value,
    odds_ratio = or,
    or_ci = ci,
    log_or = log_or,
    log_or_ci = log_ci,
    table = contingency_table,
    data = df_plot
  )

  # -----------------------------
  # Return
  # -----------------------------

  if (verbose) {

    .print_header("Fisher's Exact Test")

    .print_block("Statistics", function() {

      cat(
        "Odds Ratio = ",
        round(or, 3),
        " [",
        round(ci[1], 3), ", ",
        round(ci[2], 3),
        "]\n",
        sep = ""
      )

      cat(
        "log(OR) = ",
        round(log_or, 3),
        " [",
        round(log_ci[1], 3), ", ",
        round(log_ci[2], 3),
        "]\n",
        sep = ""
      )

      cat(
        "p = ",
        .format_pval(test$p.value),
        "\n",
        sep = ""
      )
    })
  }

  return(invisible(list(result = obj)))
}

#' Kruskal-Wallis Test with Dunn Post-hoc
#'
#' Performs the Kruskal-Wallis rank-sum test for comparing three or more
#' independent groups, followed by Dunn's post-hoc test with multiple
#' comparison adjustment.
#'
#' This function is a non-parametric alternative to one-way ANOVA and is
#' recommended when normality or homoscedasticity assumptions are violated.
#'
#' @param ... Numeric vectors representing groups, or a data frame with
#'   two or more columns (each column is treated as a group).
#' @param title Character. Plot title.
#' @param xlab Character. X-axis label.
#' @param ylab Character. Y-axis label.
#' @param style Character. Plot style. One of:
#'   \code{"boxplot"}, \code{"violin"}, \code{"mono"}, or \code{"halfeye"}.
#' @param adjust Character. Method for p-value adjustment in Dunn's test.
#'   One of \code{"bonferroni"}, \code{"holm"}, or \code{"BH"}.
#' @param help Logical. If \code{TRUE}, displays a short help message and exits.
#' @param verbose Logical. If \code{TRUE}, prints formatted statistical results
#'   to the console.
#'
#' @return Invisibly returns a list with the following components:
#'   \describe{
#'     \item{type}{Test type.}
#'     \item{H}{Kruskal-Wallis H statistic.}
#'     \item{df}{Degrees of freedom.}
#'     \item{p}{Global test p-value.}
#'     \item{epsilon_sq}{Epsilon-squared effect size.}
#'     \item{epsilon_ci}{Bootstrap confidence interval for effect size.}
#'     \item{means_sd}{Group means and standard deviations.}
#'     \item{dunn}{Dunn post-hoc results.}
#'     \item{significant_pairs}{Significant pairwise comparisons.}
#'     \item{data}{Long-format data used in the analysis.}
#'   }
#'
#' @examples
#' set.seed(123)
#'
#' n <- 25
#'
#' df <- data.frame(
#'   control    = rexp(n, rate = 1),
#'   treatment1 = rexp(n, rate = 0.6),
#'   treatment2 = rgamma(n, shape = 2, scale = 1)
#' )
#'
#' test.kruskal(df)
#'
#' @export

test.kruskal <- function(...,
                         title = "Kruskal-Wallis + Dunn",
                         xlab = "Group",
                         ylab = "Value",
                         style = c("boxplot", "violin", "mono", "halfeye"),
                         adjust = c("bonferroni", "holm", "BH"),
                         help = FALSE,
                         verbose = TRUE) {

  # Capture arguments
  args <- list(...)
  style <- match.arg(style)
  adjust <- match.arg(adjust)

  # Allow data frame input
  if (length(args) == 1 && is.data.frame(args[[1]]) && ncol(args[[1]]) >= 2) {
    groups <- lapply(args[[1]], function(col) col)
    group_names <- colnames(args[[1]])
  } else {
    groups <- args
    raw_names <- as.character(match.call(expand.dots = FALSE)$...)
    group_names <- sub("^.*\\$", "", raw_names)
  }

  # Help message
  if (help || length(groups) < 2) {
    message("
Function test.kruskal()

Description:
  Kruskal-Wallis test (non-parametric ANOVA) followed by Dunn's post-hoc test.
  Ideal for comparing three or more independent groups with non-normal data.

Example:
  set.seed(123)

n <- 25

df <- data.frame(
    control    = rexp(n, rate = 1.5),
    treatment1 = rexp(n, rate = 1),
    treatment2 = rgamma(n, shape = 2, scale = 1)
)

test.kruskal(df, style = 'violin')
")
return(invisible(NULL))
  }

  # Required packages
  required_packages <- c("ggplot2", "FSA", "dplyr", "RColorBrewer")
  lapply(required_packages, function(pkg) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
      stop(sprintf("Package '%s' is not installed.", pkg), call. = FALSE)
    }
  })

  # Data preparation
  data <- data.frame(
    value = unlist(groups),
    group = factor(
      rep(group_names, times = sapply(groups, length)),
      levels = group_names
    )
  )

  # Kruskal-Wallis test
  kruskal_res <- stats::kruskal.test(value ~ group, data = data)
  p_kruskal <- kruskal_res$p.value

  # -----------------------------
  # Effect size: Epsilon squared
  # -----------------------------

  k <- length(unique(data$group))
  n <- nrow(data)
  H <- as.numeric(kruskal_res$statistic)

  epsilon_sq <- (H - k + 1) / (n - k)

  # Bootstrap CI
  set.seed(123)

  boot_eps <- replicate(2000, {

    idx <- sample(seq_len(n), replace = TRUE)
    d_boot <- data[idx, ]

    H_boot <- suppressWarnings(
      kruskal.test(value ~ group, data = d_boot)$statistic
    )

    (H_boot - k + 1) / (n - k)
  })

  eps_ci <- quantile(boot_eps, c(0.025, 0.975), na.rm = TRUE)

  # Means and standard deviations (no automatic printing)
  mean_sd <- aggregate(
    value ~ group,
    data,
    function(x) c(mean = mean(x), sd = sd(x))
  )
  mean_sd <- do.call(data.frame, mean_sd)
  colnames(mean_sd)[2:3] <- c("mean", "sd")

  # -----------------------------
  # Dunn post-hoc test
  # -----------------------------
  suppressMessages({
    suppressWarnings({
      dunn_res <- FSA::dunnTest(
        value ~ group,
        data = data,
        method = adjust
      )
    })
  })

  dunn_df <- dunn_res$res
  significant_pairs <- subset(dunn_df, P.adj < 0.05)

  # --------------------------
  # Labels position
  # --------------------------
  sig_pairs <- significant_pairs

  if (nrow(sig_pairs) > 0) {

    comps <- strsplit(sig_pairs$Comparison, " - ")

    sig_pairs$group1 <- sapply(comps, function(x) trimws(x[1]))
    sig_pairs$group2 <- sapply(comps, function(x) trimws(x[2]))
  }

  group_levels <- levels(data$group)

  sig_pairs$x1 <- match(sig_pairs$group1, group_levels)
  sig_pairs$x2 <- match(sig_pairs$group2, group_levels)

  sig_pairs$signif <- ifelse(sig_pairs$P.adj < 0.001, "***",
                             ifelse(sig_pairs$P.adj < 0.01, "**",
                                    ifelse(sig_pairs$P.adj < 0.05, "*", "")))

  y_max <- max(data$value, na.rm = TRUE)
  y_range <- diff(range(data$value, na.rm = TRUE))

  step <- 0.08 * y_range

  sig_pairs$y <- y_max + seq_len(nrow(sig_pairs)) * step


  # --------------------------
  # Colors
  # --------------------------
  # Vivid colors
  vivid_colors <- scales::hue_pal()(length(unique(data$group)))

  # mono
  n <- length(groups)

  mono_colors <- gray.colors(
    n,
    start = 0.9,
    end = 0.1
  )

  # --------------------------
  # STYLE 1: Boxplot + jitter
  # --------------------------
  if (style == "boxplot") {
    g <- ggplot2::ggplot(data, ggplot2::aes(x = group, y = value, fill = group)) +
      ggplot2::geom_boxplot(alpha = 0.7, outlier.shape = NA, width = 0.7, linewidth = 0.7) +
      ggplot2::geom_jitter(width = 0.1, alpha = 0.5, color = "grey25") +
      ggplot2::labs(
        title = title,
        subtitle = .build_subtitle_kw(p_kruskal, epsilon_sq),
        x = "",
        y = ylab
      ) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(
        legend.position = "none",
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 12)
      )
  }

  # --------------------------
  # STYLE 2: Violin + minimalist boxplot
  # --------------------------
  if (style == "violin") {
    g <- ggplot2::ggplot(data, ggplot2::aes(x = group, y = value, fill = group)) +
      ggplot2::geom_violin(
        trim = FALSE,
        alpha = 0.6,
        color = NA,
        adjust = 0.6
      ) +
      ggplot2::geom_boxplot(
        width = 0.18,
        outlier.shape = NA,
        color = "gray20",
        linewidth = 0.4
      ) +
      ggplot2::geom_point(
        position = ggplot2::position_jitter(width = 0.1),
        alpha = 0.2,
        size = 1.8,
        color = "gray25"
      ) +
      ggplot2::labs(
        title = title,
        subtitle = .build_subtitle_kw(p_kruskal, epsilon_sq),
        x = "",
        y = ylab
      ) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(
        legend.position = "none",
        axis.text.x = ggplot2::element_text(angle = 45,
                                            hjust = 1,
                                            size = 12)
      )
  }

  # --------------------------
  # STYLE 3: monochrome premium
  # --------------------------
  if (style == "mono") {
    g <- ggplot2::ggplot(data, ggplot2::aes(x = group, y = value, fill = group)) +
      ggplot2::geom_boxplot(alpha = 0.7,outlier.shape = NA, width = 0.7, linewidth = 0.7, color = "black") +
      ggplot2::geom_jitter(width = 0.1, alpha = 0.2, color = "grey25") +
      ggplot2::labs(
        title = title,
        subtitle = .build_subtitle_kw(p_kruskal, epsilon_sq),
        x = "",
        y = ylab
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::scale_fill_manual(values = mono_colors) +
      ggplot2::theme(legend.position = "none",
                     axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 12))
   }

  # --------------------------
  # STYLE 4: Half-eye (ggdist)
  # --------------------------
  if (style == "halfeye") {
    if (!requireNamespace("ggdist", quietly = TRUE)) {
      stop("Style = 'halfeye' requires the 'ggdist' package.")
    }

    g <- ggplot2::ggplot(data, ggplot2::aes(x = group, y = value, fill = group)) +
      ggdist::stat_halfeye(
        alpha = .6,
        trim = FALSE,
        adjust = 0.6,
        width = 0.6,
        .width = c(0.5, 0.8, 0.95),
        justification = -0.2,
        slab_color = "gray20",
        interval_color = "gray20"
      ) +
      ggplot2::geom_point(
        position = ggplot2::position_nudge(x = 0.15),
        size = 1.1,
        alpha = 0.4,
        color = "black"
      ) +
      ggdist::stat_pointinterval(
        position = ggplot2::position_nudge(x = 0.2),
        point_color = "black",
        interval_color = "black",
        .width = 0.95
      ) +
      ggplot2::labs(
        title = title,
        subtitle = .build_subtitle_kw(p_kruskal, epsilon_sq),
        x = "",
        y = ylab
      ) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(
        legend.position = "none",
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 12)
      )
  }

  # --------------------------
  # Annotation and print
  # --------------------------
  g <- g + .add_significance(sig_pairs, y_range)

  print(g)

  # --------------------------
  # Return
  # --------------------------
  invisible(list(
    type = "Kruskal-Wallis",
    H = H,
    df = k - 1,
    p = p_kruskal,
    epsilon_sq = epsilon_sq,
    epsilon_ci = eps_ci,
    means_sd = mean_sd,
    dunn = dunn_df,
    significant_pairs = significant_pairs,
    data = data
  ))

  # -----------------------------
  # Output
  # -----------------------------
  if (verbose) {

    .print_header("Kruskal-Wallis")

    .print_block("Statistics", function() {

      cat(
        "H statistic = ",
        round(H, 3),
        " | df = ",
        k - 1,
        " | p = ",
        .format_pval(p_kruskal),
        "\n",
        sep = ""
      )

      cat(
        "Epsilon squared = ",
        round(epsilon_sq, 3),
        " [",
        round(eps_ci[1], 3), ", ",
        round(eps_ci[2], 3),
        "]\n",
        sep = ""
      )
    })

    # --- Post-hoc output ---

    .print_header(paste0("Post-hoc: Dunn (", adjust, ")"))

    .print_block("Significant comparisons", function() {

      if (nrow(significant_pairs) == 0) {

        cat("No significant comparisons (p < 0.05)\n")

      } else {

        for (i in seq_len(nrow(significant_pairs))) {

          r <- significant_pairs[i, ]

          comps <- unlist(strsplit(r$Comparison, " - "))

          g1 <- trimws(comps[1])
          g2 <- trimws(comps[2])

          cat(g1, " vs ", g2, "\n", sep = "")

          cat(
            "Z = ",
            round(r$Z, 3),
            "\n",
            sep = ""
          )

          cat(
            "p (adj) = ",
            .format_pval(r$P.adj),
            "\n\n",
            sep = ""
          )
        }
      }
    })
  }
}

# Declare global variables to avoid NOTE: no visible binding
# ---------------------------------------------------------------------------------
if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    # Internal variables
    "group", "id", "ind", "n", "p adj", "p.adj", "P.adj", "p_adj",
    "prop", "value", "values", "x1", "x2", "y", "signif",

    # Standard variables
    "group", "value"
  ))
}

# ============================
# Auxiliary bootstrap functions
# ============================

#' @keywords internal
.boot_one_sample <- function(
    x,
    stat_fun,
    B = 2000
) {

  boot_stat <- replicate(B, {

    x_b <- sample(x, length(x), replace = TRUE)
    stat_fun(x_b)

  })

  boot_stat <- boot_stat[is.finite(boot_stat)]

  ci <- quantile(boot_stat, c(0.025, 0.975), na.rm = TRUE)

  list(
    stat = stat_fun(x),
    ci_low = ci[1],
    ci_high = ci[2],
    boot = boot_stat
  )
}


#' @keywords internal
.boot_two_sample <- function(x, y, stat_fun, B = 2000, conf = 0.95) {

  x <- x[!is.na(x)]
  y <- y[!is.na(y)]

  boot_vals <- replicate(B, {

    xb <- sample(x, length(x), replace = TRUE)
    yb <- sample(y, length(y), replace = TRUE)

    stat_fun(xb, yb)

  })

  alpha <- (1 - conf) / 2

  ci <- quantile(
    boot_vals,
    probs = c(alpha, 1 - alpha),
    na.rm = TRUE
  )

  list(
    boot = boot_vals,
    ci_low = ci[1],
    ci_high = ci[2]
  )
}


#' @keywords internal
.boot_anova_omega <- function(data, group, value, B = 2000, conf = 0.95) {

  data <- data[!is.na(data[[value]]), ]

  groups <- unique(data[[group]])

  boot_vals <- replicate(B, {

    boot_data <- do.call(
      rbind,
      lapply(groups, function(g) {

        sub <- data[data[[group]] == g, ]

        sub[sample(nrow(sub), replace = TRUE), ]

      })
    )

    fit <- stats::aov(
      reformulate(group, value),
      data = boot_data
    )

    tab <- summary(fit)[[1]]

    ssb <- tab[1, "Sum Sq"]
    ssw <- tab[nrow(tab), "Sum Sq"]

    dfb <- tab[1, "Df"]
    msw <- tab[nrow(tab), "Mean Sq"]

    sst <- ssb + ssw

    (ssb - dfb * msw) / (sst + msw)

  })

  alpha <- (1 - conf) / 2

  ci <- quantile(
    boot_vals,
    c(alpha, 1 - alpha),
    na.rm = TRUE
  )

  list(
    omega = mean(boot_vals, na.rm = TRUE),
    ci_low = ci[1],
    ci_high = ci[2],
    boot = boot_vals
  )
}


#' @keywords internal
.boot_cramers_v <- function(tab, B = 2000, conf = 0.95) {

  df <- as.data.frame(as.table(tab))
  colnames(df) <- c("x", "y", "n")

  expanded <- df[rep(seq_len(nrow(df)), df$n), 1:2]

  boot_vals <- replicate(B, {

    idx <- sample(nrow(expanded), replace = TRUE)

    tb <- table(
      expanded$x[idx],
      expanded$y[idx]
    )

    .cramers_v(tb)

  })

  alpha <- (1 - conf) / 2

  ci <- quantile(
    boot_vals,
    c(alpha, 1 - alpha),
    na.rm = TRUE
  )

  list(
    v = mean(boot_vals, na.rm = TRUE),
    ci_low = ci[1],
    ci_high = ci[2],
    boot = boot_vals
  )
}

#' Chi-square test
#'
#' Applies the Pearson chi-square test or Fisher's exact test to assess association
#' between two categorical variables.
#'
#' @param x Categorical vector or data frame with two columns (group 1 and group 2).
#' @param y Categorical vector (group 2). Required if x is a vector.
#' @param title Plot title (string). Default: "Chi-square Test".
#' @param xlab X-axis label in the plot (string). Default: NULL (uses variable name).
#' @param ylab Y-axis label in the plot (string). Default: "Proportion".
#' @param style Plot style generated by the function.
#' @param show_table Logical. If TRUE, prints the contingency table to the console.
#'        Default: TRUE.
#' @param help Logical. If TRUE, displays a detailed explanation of the function.
#'        Default: FALSE.
#' @param verbose Logical. If TRUE, prints messages about the test and expected
#'        frequencies. Default: TRUE.
#' @return Test result and contingency table.
#' @export
#'
#' @examples
#'
#' data <- data.frame(
#'   control = c(rep("healthy", 50), rep("sick", 150)),
#'   treatment = c(rep("healthy", 100), rep("sick", 100))
#' )
#' test.chi(data)

test.chi <- function(x, y = NULL,
                     title = "Chi-square Test",
                     xlab = NULL,
                     ylab = "Proportion",
                     style = c("stacked", "barplot", "mosaic", "pie"),
                     show_table = TRUE,
                     help = FALSE,
                     verbose = TRUE) {

  style <- match.arg(style)

  # Help
  if (help || missing(x)) {
    if (verbose) {
      message(
        "Function test.chi()

Description:
  Performs Pearson's Chi-square test to evaluate association between
  two categorical variables.

When to use:
  - Two categorical variables with large sample sizes
  - Contingency tables may have more than 2 categories per variable
  - Expected cell frequencies should be >= 5

Limitations:
  - Not recommended for small tables; use 'fisher_test()' instead.

Example:
  data <- data.frame(
    control = c(rep('healthy', 50), rep('sick', 150)),
    treatment = c(rep('healthy', 100), rep('sick', 100))
  )
  test.chi(data)
"
      )
    }
    return(invisible(NULL))
  }

  # Required packages
  required_packages <- c("ggplot2", "dplyr", "tidyr", "vcd")
  for (pkg in required_packages) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
      stop(
        paste0(
          "Package '", pkg,
          "' is not installed. Install it with install.packages('", pkg, "')"
        ),
        call. = FALSE
      )
    }
  }

  # Case 1: x is a data frame with two columns
  if (is.data.frame(x)) {
    if (ncol(x) != 2) {
      stop(
        "The data frame must contain exactly two categorical columns.",
        call. = FALSE
      )
    }

    # Store original column names
    column_names <- colnames(x)

    # Convert to long format
    data_long <- tidyr::pivot_longer(
      x,
      cols = tidyselect::everything(),
      names_to = "group",
      values_to = "category"
    )

    # Preserve original column names as factor levels
    data_long$group <- factor(
      data_long$group,
      levels = column_names,
      labels = column_names
    )

    group <- data_long$group
    category <- data_long$category

    name_x <- column_names[1]
    name_y <- column_names[2]

  } else {
    # Case 2: x and y are vectors
    if (is.null(y)) {
      stop("Argument 'y' must be provided if 'x' is a vector.", call. = FALSE)
    }
    if (length(x) != length(y)) {
      stop("Both variables must have the same length.", call. = FALSE)
    }

    group <- x
    category <- y

    name_x <- deparse(substitute(x))
    name_y <- deparse(substitute(y))
    name_x <- sub(".*\\$", "", name_x)
    name_y <- sub(".*\\$", "", name_y)
  }

  if (is.null(xlab)) xlab <- name_x
  if (is.null(ylab)) ylab <- "Proportion"

  # Contingency table
  contingency_table <- table(group, category)

  if (verbose && show_table) {
    message("Observed contingency table:")
    print(contingency_table)
  }

  # Chi-square test
  test <- suppressWarnings(stats::chisq.test(contingency_table))
  expected_freq <- test$expected

  small_exp <- any(expected_freq < 5)

  if (verbose && small_exp) {
    warning(
      "Some expected frequencies < 5. Chi-square approximation may be unreliable. ",
      "Consider Fisher's exact test."
    )
  }

  # -----------------------------
  # Effect size: Cramér's V
  # -----------------------------

  v <- .cramers_v(contingency_table)
  boot_v <- .boot_cramers_v(contingency_table)


  # -----------------------------
  # Data preparation for plotting
  # -----------------------------
  df_plot <- data.frame(group = group, category = category)

  df_prop <- df_plot |>
    dplyr::group_by(group, category) |>
    dplyr::summarise(n = dplyr::n(), .groups = "drop") |>
    dplyr::group_by(group) |>
    dplyr::mutate(prop = n / sum(n))

  # Subtitle
  subtitle_text <- .make_subtitle_chi(
    cramers_v = v,
    p_value = test$p.value,
    small_expected = small_exp
  )

  # Labels and colors
  vivid_colors <- scales::hue_pal()(length(unique(df_prop$group)))

  # --------------------------
  # STYLE 1 (Stacked bar plot)
  # --------------------------
  if (style == "stacked") {
    g <- ggplot2::ggplot(
      df_prop,
      ggplot2::aes(x = group, y = prop, fill = category)
    ) +
      ggplot2::geom_bar(stat = "identity", color = NA) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::labs(
        title = title,
        subtitle = subtitle_text,
        x = "",
        y = ylab,
        fill = name_y
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(
        legend.position = "right",
        axis.text.x = ggplot2::element_text(
          angle = 45, hjust = 1, size = 12
        )
      )
  }

  # --------------------------
  # STYLE 2 (Side-by-side bars)
  # --------------------------
  if (style == "barplot") {
    g <- ggplot2::ggplot(
      df_prop,
      ggplot2::aes(x = group, y = prop, fill = category)
    ) +
      ggplot2::geom_bar(
        stat = "identity",
        position = ggplot2::position_dodge(width = 0.8)
      ) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::labs(
        title = title,
        subtitle = subtitle_text,
        x = "",
        y = ylab,
        fill = name_y
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(
        legend.position = "right",
        axis.text.x = ggplot2::element_text(
          angle = 45, hjust = 1, size = 12
        )
      )
  }

  # --------------------------
  # STYLE 3 (Mosaic plot)
  # --------------------------
  if (style == "mosaic") {
    if (!requireNamespace("vcd", quietly = TRUE)) {
      stop(
        "Style = 'mosaic' requires the 'vcd' package. Install it with install.packages('vcd')"
      )
    }

    vcd::mosaic(
      contingency_table,
      shade = TRUE,
      legend = TRUE,
      main = paste0(title, "\n", subtitle_text)
    )
  }

  # --------------------------
  # STYLE 4 (Pie chart)
  # --------------------------
  if (style == "pie") {

    g <- ggplot2::ggplot(
      df_prop,
      ggplot2::aes(x = "", y = prop, fill = category)
    ) +
      ggplot2::geom_bar(stat = "identity", width = 1) +
      ggplot2::coord_polar("y") +
      ggplot2::facet_wrap(~ group) +
      ggplot2::scale_fill_manual(values = vivid_colors) +
      ggplot2::theme_void(base_size = 12) +
      ggplot2::labs(
        title = title,
        subtitle = subtitle_text,
        fill = name_y
      ) +
      ggplot2::theme(
        plot.title = ggplot2::element_text(
          hjust = 0.5,
          size = 14
        ),
        plot.subtitle = ggplot2::element_text(
          hjust = 0.5,
          size = 11,
          margin = ggplot2::margin(b = 10)
        ),
        strip.text = ggplot2::element_text(
          size = 12
        )
      )
  }

  if (style != "mosaic") print(g)

  # --------------------------
  # Output
  # --------------------------
  obj <- list(
    type = "Chi-square",
    statistic = as.numeric(test$statistic),
    df = as.numeric(test$parameter),
    p = test$p.value,
    cramers_v = v,
    cramers_ci = c(
      boot_v$ci_low,
      boot_v$ci_high
    ),
    expected = expected_freq,
    table = contingency_table,
    small_expected = small_exp,
    data = df_plot
  )

  # -----------------------------
  # Return
  # -----------------------------

  if (verbose) {

    .print_header("Chi-square Test")

    .print_block("Statistics", function() {

      cat(
        "Chi-square = ",
        round(test$statistic, 3),
        " | df = ",
        test$parameter,
        " | p = ",
        .format_pval(test$p.value),
        "\n",
        sep = ""
      )

      cat(
        "Cramer's V = ",
        round(v, 3),
        " [",
        round(boot_v$ci_low, 3), ", ",
        round(boot_v$ci_high, 3),
        "]\n",
        sep = ""
      )

      if (small_exp) {
        cat(
          "Note: Some expected counts < 5 (interpret with caution)\n"
        )
      }

    })
  }

  return(invisible(list(result = obj)))

  }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pre_Test.R
\name{pre.test}
\alias{pre.test}
\title{pre.test() — Automatic statistical test suggestion}
\usage{
pre.test(..., alpha = 0.05, help = FALSE, verbose = TRUE)
}
\arguments{
\item{...}{Two or more vectors (numeric or categorical), or a data.frame with >= 2 columns}

\item{alpha}{Significance level. Default = 0.05}

\item{help}{Logical. If TRUE, shows detailed help}

\item{verbose}{Logical. If TRUE, prints informative messages}
}
\value{
Invisible list with normality results, homogeneity or contingency table,
and test recommendation
}
\description{
Automatically identifies whether the input data are numeric or categorical
and suggests the most appropriate statistical test.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Paired_T_test.R
\name{test.tpaired}
\alias{test.tpaired}
\title{Paired t-test with advanced visualizations}
\usage{
test.tpaired(
  ...,
  title = "Paired t-test",
  xlab = "",
  ylab = "Value",
  style = c("boxplot", "violin", "mono", "halfeye"),
  connect = TRUE,
  help = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{...}{Two numeric vectors of equal length, or
a data frame with exactly two numeric columns.}

\item{title}{Plot title.}

\item{xlab}{X-axis label.}

\item{ylab}{Y-axis label.}

\item{style}{Plot style:
\itemize{
\item \code{1} Premium boxplot
\item \code{2} Violin + minimal boxplot
\item \code{3} mono
\item \code{4} Half-eye (ggdist)
}}

\item{connect}{Logical. If TRUE, connects paired observations.}

\item{help}{If TRUE, displays detailed help.}

\item{verbose}{If TRUE, prints progress messages.}
}
\value{
An invisible list containing:
\describe{
\item{summary_table}{Group means and standard deviations}
\item{test_result}{t-test result object (stats::t.test)}
\item{data}{Data frame used for plotting}
\item{plot}{ggplot2 object}
}
}
\description{
Performs a paired t-test between two numeric vectors (e.g., before vs after)
or between two numeric columns of a data frame.
Includes four visualization styles (boxplot, violin, mono, and half-eye).
}
\examples{
before <- c(13, 12, 15, 14)
after  <- c(9, 11, 10, 10)
test.tpaired(before, after)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autotestR-package.R
\docType{package}
\name{autotestR}
\alias{autotestR-package}
\alias{autotestR}
\title{autotestR}
\description{
designed to simplify the execution of the main statistical tests commonly used in the life sciences.
It provides user-friendly functions that automatically generate plots and clear explanations,
making statistical analysis more accessible for researchers and students.
}
\section{Main features}{
\itemize{
\item t test (independent and paired)
\item Mann–Whitney test (Wilcoxon rank-sum)
\item Multiple group comparison (t test or Mann–Whitney)
\item Chi-squared test and Fisher’s exact test
\item One-way ANOVA with Tukey HSD post hoc test
\item Kruskal–Wallis test with Dunn post hoc test
\item Pearson, Spearman, and Kendall correlation tests with automatic plots
\item Diagnostic function that suggests the most appropriate statistical test
\item Intuitive plots fully integrated into the functions
}
\subsection{Basic usage}{

library(autotestR)
}
}

\section{Independent t test}{
group1 <- rnorm(30, 10, 2)
group2 <- rnorm(30, 12, 2)
test.t(group1, group2)
}

\section{Chi-squared test}{
var1 <- sample(c("A", "B"), 100, replace = TRUE)
var2 <- sample(c("Yes", "No"), 100, replace = TRUE)
test.chi(var1, var2)
}

\section{Multiple test (t test or Mann–Whitney)}{
df <- data.frame(
control   = rnorm(30, 10),
treatment = rnorm(30, 12),
test1     = rnorm(30, 11),
test2     = rnorm(30, 15)
)
test.tmulti(df)
}

\section{ANOVA with post hoc test}{
g1 <- rnorm(20, 5)
g2 <- rnorm(20, 7)
g3 <- rnorm(20, 6)
test.anova(g1, g2, g3)
}

\section{Correlation test}{
x <- rnorm(30)
y <- x + rnorm(30, 0, 1)
test.correlation(x, y)
}

\author{
\strong{Maintainer}: Luiz Garcia \email{luiz.cardoso@ufpr.br} (\href{https://orcid.org/0000-0002-9616-0927}{ORCID})

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/T_test.R
\name{test.t}
\alias{test.t}
\title{Student's t-test (with automatic diagnostics)}
\usage{
test.t(
  ...,
  title = TRUE,
  title_text = "t-test",
  xlab = "Group",
  ylab = "Value",
  style = c("boxplot", "violin", "mono", "halfeye"),
  help = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{...}{Two numeric vectors or a data frame with exactly two columns.}

\item{title}{Logical. If true, return a plot entitled.}

\item{title_text}{Plot title (string). Default: "t-test".}

\item{xlab}{X-axis label in the plot (string). Default: "Group".}

\item{ylab}{Y-axis label in the plot (string). Default: "Value".}

\item{style}{Plot aesthetic generated by the function.}

\item{help}{Logical. If TRUE, shows a detailed explanation of the function.
Default: FALSE.}

\item{verbose}{Logical. If TRUE, prints detailed messages. Default: TRUE.}
}
\value{
Invisible list with summary, test test_result, method and (optionally) plot.
}
\description{
Performs Student's t-test to compare means between two independent groups,
with automatic checks for normality and homogeneity of variances.
If assumptions are violated, the Mann-Whitney test is automatically applied
(without generating a plot).
}
\examples{
set.seed(123)
df <- data.frame(
  control = rnorm(30, 10),
  treatment = rnorm(30, 15)
)
test.t(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Correlation_test.R
\name{test.correlation}
\alias{test.correlation}
\alias{print.test.correlation}
\title{Correlation Analysis with Automatic Method Selection}
\usage{
test.correlation(
  x,
  y = NULL,
  method = "auto",
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  style = c("simple", "inference", "structure", "density", "distribution"),
  plot_normality = FALSE,
  help = FALSE,
  verbose = TRUE
)

\method{print}{test.correlation}(x, ...)
}
\arguments{
\item{x}{Numeric vector or data frame with exactly two numeric columns.}

\item{y}{Numeric vector (optional if \code{x} is a data frame).}

\item{method}{Correlation method: \code{"auto"} (default), \code{"pearson"},
\code{"spearman"}, or \code{"kendall"}.}

\item{main}{Plot title. If \code{NULL}, an automatic title is generated.
If \code{FALSE}, no title is displayed.}

\item{xlab}{Label for the x-axis. If \code{NULL}, a label is inferred
from the input. If \code{FALSE}, no label is shown.}

\item{ylab}{Label for the y-axis. If \code{NULL}, a label is inferred
from the input. If \code{FALSE}, no label is shown.}

\item{style}{Plot style: \code{"simple"} (default), \code{"inference"},
\code{"structure"}, \code{"density"}, or \code{"distribution"}.}

\item{plot_normality}{Logical. If \code{TRUE}, displays QQ-plots for
normality assessment.}

\item{help}{Logical. If \code{TRUE}, displays a detailed usage guide.}

\item{verbose}{Logical. If \code{TRUE}, prints information about method
selection and diagnostics.}

\item{...}{Additional arguments passed to other print methods (currently ignored)}
}
\value{
An object of class \code{"test.correlation"} containing:
\itemize{
\item \code{call}: Matched function call.
\item \code{data}: Input data (if \code{n <= 5000}).
\item \code{n}: Sample size.
\item \code{method}: Selected correlation method.
\item \code{estimate}: Correlation coefficient.
\item \code{effect}: Effect size information (r, rho, or tau, and r² when applicable).
\item \code{conf.int}: Confidence interval.
\item \code{p.value}: P-value of the test.
\item \code{interpretation}: Qualitative interpretation of effect size.
\item \code{diagnostics}: Normality, ties, and outlier diagnostics.
\item \code{decision}: Automatic selection rationale.
\item \code{htest}: Underlying \code{stats::cor.test} object.
\item \code{plot}: \code{ggplot2} object.
}
}
\description{
Performs correlation analysis between two numeric variables using Pearson,
Spearman, or Kendall methods. When \code{method = "auto"}, the function
automatically selects the most appropriate method based on normality,
presence of ties, and proportion of outliers.
}
\details{
In addition to hypothesis testing, the function provides diagnostic
information, effect size interpretation, and publication-ready
visualizations.

Method selection in automatic mode follows these rules:
\itemize{
\item If more than 5\\% of observations are classified as outliers,
Kendall's tau is used.
\item If both variables are normally distributed and no ties are present,
Pearson's correlation is used.
\item Otherwise, Spearman's rank correlation is applied.
}

Normality is assessed using Shapiro-Wilk (n <= 50), Anderson-Darling
(50 < n <= 300), or Kolmogorov-Smirnov (n > 300) tests.
}
\examples{
# Pearson (approximately normal data)
set.seed(123)
x <- rnorm(50, sd = 0.1)
y <- x + rnorm(50, sd = 0.1)
test.correlation(x, y)

# Spearman (non-normal data)
set.seed(123)
x <- runif(300)
y <- log(x + 0.1) + rnorm(300, sd = 0.5)
test.correlation(x, y)

# Kendall (ties and outliers)
set.seed(123)
x <- runif(1000, 1, 100)
y <- sin(x) * 30 + rnorm(1000, 0, 10)
x[sample(1:500, 50)] <- 50
y[sample(1:500, 50)] <- 0
x_out <- runif(100, 10, 20)
y_out <- runif(100, 80, 120)
x <- c(x, x_out)
y <- c(y, y_out)
test.correlation(x, y)

}
\seealso{
\code{\link[stats]{cor.test}},
\code{\link[ggplot2]{ggplot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Multi_T_test.R
\name{test.tmulti}
\alias{test.tmulti}
\title{Multiple pairwise comparisons with diagnostics}
\usage{
test.tmulti(
  ...,
  comparisons = NULL,
  title = "Multiple comparisons (t / MW)",
  xlab = "",
  ylab = "Value",
  style = c("boxplot", "violin", "mono", "halfeye"),
  p_adjust = c("none", "holm", "BH", "bonferroni"),
  help = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{...}{Numeric vectors or a data.frame with groups in columns.}

\item{comparisons}{List of character vectors specifying pairwise
comparisons (e.g. list(c("A","B"), c("B","C"))).
If NULL, all pairwise combinations are used.}

\item{title}{Plot title.}

\item{xlab}{X-axis label.}

\item{ylab}{Y-axis label.}

\item{style}{Plot style. One of "boxplot", "violin",
"mono", or "halfeye".}

\item{p_adjust}{Method for multiple testing correction.
One of "none", "holm", "BH", "bonferroni".}

\item{help}{Logical. If TRUE, prints usage examples.}

\item{verbose}{Logical. If TRUE, prints results and plots.}
}
\value{
A list with:
\describe{
\item{results}{A tibble with test results.}
\item{plot}{A ggplot object.}
\item{data_long}{Long-format data used for plotting.}
}
}
\description{
Performs multiple pairwise comparisons using Student's t-test
or Mann-Whitney test, with automatic diagnostics, effect sizes,
confidence intervals, multiple testing correction and visualization.
}
\details{
Normality is assessed using Shapiro-Wilk tests and homogeneity
of variances using Levene's test. If assumptions are met, a
pooled-variance t-test is used. Otherwise, the Mann-Whitney test
is applied with bootstrap confidence intervals.

Effect sizes:
\itemize{
\item Cohen's d for t-tests
\item Rank-biserial correlation for Mann-Whitney
}
}
\examples{
df <- data.frame(
  control   = rnorm(30, 10),
  treatment = rnorm(30, 12),
  test1     = rnorm(30, 11),
  test2     = rnorm(30, 15)
)

test.tmulti(
  df,
  comparisons = list(
    c("control", "treatment"),
    c("treatment", "test1")
  )
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Chi_test.R
\name{test.chi}
\alias{test.chi}
\title{Chi-square test}
\usage{
test.chi(
  x,
  y = NULL,
  title = "Chi-square Test",
  xlab = NULL,
  ylab = "Proportion",
  style = c("stacked", "barplot", "mosaic", "pie"),
  show_table = TRUE,
  help = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{x}{Categorical vector or data frame with two columns (group 1 and group 2).}

\item{y}{Categorical vector (group 2). Required if x is a vector.}

\item{title}{Plot title (string). Default: "Chi-square Test".}

\item{xlab}{X-axis label in the plot (string). Default: NULL (uses variable name).}

\item{ylab}{Y-axis label in the plot (string). Default: "Proportion".}

\item{style}{Plot style generated by the function.}

\item{show_table}{Logical. If TRUE, prints the contingency table to the console.
Default: TRUE.}

\item{help}{Logical. If TRUE, displays a detailed explanation of the function.
Default: FALSE.}

\item{verbose}{Logical. If TRUE, prints messages about the test and expected
frequencies. Default: TRUE.}
}
\value{
Test result and contingency table.
}
\description{
Applies the Pearson chi-square test or Fisher's exact test to assess association
between two categorical variables.
}
\examples{

data <- data.frame(
  control = c(rep("healthy", 50), rep("sick", 150)),
  treatment = c(rep("healthy", 100), rep("sick", 100))
)
test.chi(data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Kruskal_test.R
\name{test.kruskal}
\alias{test.kruskal}
\title{Kruskal-Wallis Test with Dunn Post-hoc}
\usage{
test.kruskal(
  ...,
  title = "Kruskal-Wallis + Dunn",
  xlab = "Group",
  ylab = "Value",
  style = c("boxplot", "violin", "mono", "halfeye"),
  adjust = c("bonferroni", "holm", "BH"),
  help = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{...}{Numeric vectors representing groups, or a data frame with
two or more columns (each column is treated as a group).}

\item{title}{Character. Plot title.}

\item{xlab}{Character. X-axis label.}

\item{ylab}{Character. Y-axis label.}

\item{style}{Character. Plot style. One of:
\code{"boxplot"}, \code{"violin"}, \code{"mono"}, or \code{"halfeye"}.}

\item{adjust}{Character. Method for p-value adjustment in Dunn's test.
One of \code{"bonferroni"}, \code{"holm"}, or \code{"BH"}.}

\item{help}{Logical. If \code{TRUE}, displays a short help message and exits.}

\item{verbose}{Logical. If \code{TRUE}, prints formatted statistical results
to the console.}
}
\value{
Invisibly returns a list with the following components:
\describe{
\item{type}{Test type.}
\item{H}{Kruskal-Wallis H statistic.}
\item{df}{Degrees of freedom.}
\item{p}{Global test p-value.}
\item{epsilon_sq}{Epsilon-squared effect size.}
\item{epsilon_ci}{Bootstrap confidence interval for effect size.}
\item{means_sd}{Group means and standard deviations.}
\item{dunn}{Dunn post-hoc results.}
\item{significant_pairs}{Significant pairwise comparisons.}
\item{data}{Long-format data used in the analysis.}
}
}
\description{
Performs the Kruskal-Wallis rank-sum test for comparing three or more
independent groups, followed by Dunn's post-hoc test with multiple
comparison adjustment.
}
\details{
This function is a non-parametric alternative to one-way ANOVA and is
recommended when normality or homoscedasticity assumptions are violated.
}
\examples{
set.seed(123)

n <- 25

df <- data.frame(
  control    = rexp(n, rate = 1),
  treatment1 = rexp(n, rate = 0.6),
  treatment2 = rgamma(n, shape = 2, scale = 1)
)

test.kruskal(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/U_test.R
\name{test.u}
\alias{test.u}
\title{Mann-Whitney U Test}
\usage{
test.u(
  ...,
  title = "Mann-Whitney Test",
  xlab = "Group",
  ylab = "Value",
  style = c("boxplot", "violin", "mono", "halfeye"),
  help = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{...}{Two numeric vectors or a data.frame with two numeric columns.}

\item{title}{Plot title. Default: "Mann-Whitney Test".}

\item{xlab}{Label for x-axis. Default: "Group".}

\item{ylab}{Label for y-axis. Default: "Value".}

\item{style}{Plot aesthetic style.}

\item{help}{Logical. If TRUE, prints a detailed explanation. Default: FALSE.}

\item{verbose}{Logical. If TRUE, prints detailed messages. Default: TRUE.}
}
\value{
Invisible list with:
\describe{
\item{summary}{Group-wise statistical summary}
\item{test}{Test result (htest object)}
\item{plot}{ggplot2 visualization object}
}
}
\description{
Performs the Mann-Whitney (Wilcoxon rank-sum) test for comparing two independent groups,
with statistical summary and graphical visualization.
}
\examples{
x <- c(1, 3, 5, 6)
y <- c(7, 8, 9, 12)
data <- data.frame(groupA = x, groupB = y)
test.u(data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Fisher_test.R
\name{test.fisher}
\alias{test.fisher}
\title{Fisher's Exact Test}
\usage{
test.fisher(
  x,
  y = NULL,
  title = "Fisher's Exact Test",
  xlab = NULL,
  ylab = "Proportion",
  style = c("stacked", "barplot", "mosaic", "pie"),
  show_table = TRUE,
  help = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{x}{Categorical vector or data frame with two columns.}

\item{y}{Categorical vector (required if x_var is a vector).}

\item{title}{Plot title (string). Default: "Fisher's Exact Test"}

\item{xlab}{Name of the x-axis in the plot (string). Default: NULL (uses variable name)}

\item{ylab}{Name of the y-axis in the plot (string). Default: "Proportion"}

\item{style}{Plot style. Controls the visualization type}

\item{show_table}{Logical. If TRUE, prints the contingency table to the console. Default: TRUE}

\item{help}{Logical. If TRUE, shows detailed function explanation. Default: FALSE}

\item{verbose}{Logical. If TRUE, prints detailed test messages. Default: TRUE}
}
\value{
Invisible object containing the Fisher test result.
}
\description{
Performs Fisher's Exact Test using two categorical vectors or a data frame with two columns,
constructing a contingency table and optionally generating graphical visualizations.
}
\examples{
data <- data.frame(control = c("healthy","healthy","sick","sick","sick"),
                   treatment = c("healthy","healthy","healthy","healthy","sick"))
test.fisher(data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ANOVA_test.R
\name{test.anova}
\alias{test.anova}
\title{ANOVA test with automated assumption checking}
\usage{
test.anova(
  ...,
  title = "ANOVA/Tukey HSD",
  xlab = "Group",
  ylab = "Value",
  style = c("boxplot", "violin", "mono", "halfeye"),
  help = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{...}{Vectors or a data.frame with >= 2 columns.}

\item{title}{Plot title.}

\item{xlab}{X-axis label.}

\item{ylab}{Y-axis label.}

\item{style}{Aesthetic style of the generated plot.}

\item{help}{If TRUE, shows help.}

\item{verbose}{If TRUE, shows detailed messages.}
}
\value{
An \code{aov} object or a recommendation message.
}
\description{
Performs ANOVA (and Tukey HSD) if data meet normality and homogeneity assumptions.
Otherwise, automatically recommends Kruskal-Wallis/Dunn.
}
\examples{
df <- data.frame(
  control = rnorm(30, 10),
  treatment = rnorm(30, 12),
  test = rnorm(30, 11)
)
test.anova(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Interaction_test.R
\name{test.interaction}
\alias{test.interaction}
\alias{print.test.interaction}
\title{Test for interaction between a continuous predictor and a grouping variable}
\usage{
test.interaction(
  x,
  y,
  by,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  plot = TRUE,
  style = c("clean", "CI", "facet"),
  conf.level = 0.95,
  verbose = TRUE,
  help = FALSE
)

\method{print}{test.interaction}(x, ...)
}
\arguments{
\item{x}{Numeric vector representing the continuous predictor.}

\item{y}{Numeric vector representing the continuous outcome.}

\item{by}{Grouping variable defining the interaction. Must be coercible
to a factor with at least two levels.}

\item{title}{Optional title title for the plot.}

\item{xlab}{Optional x-axis label.}

\item{ylab}{Optional y-axis label.}

\item{plot}{Logical. Should a plot be generated?}

\item{style}{Plot style. One of \code{"clean"}, \code{"CI"} or \code{"facet"}.}

\item{conf.level}{Confidence level for the interaction interval
(default: 0.95).}

\item{verbose}{Logical. If TRUE, prints detailed messages. Default: TRUE.}

\item{help}{Logical. If TRUE, shows a detailed explanation of the function.
Default: FALSE.}

\item{...}{Additional arguments passed to other print methods (currently ignored)}
}
\value{
An object of class \code{"test.interaction"} containing:
\itemize{
\item \code{model}: the fitted linear model,
\item \code{interaction}: estimated interaction effects with confidence intervals,
\item \code{plot}: a \code{ggplot} object (if \code{plot = TRUE}).
}
}
\description{
Fits a linear model of the form \code{y ~ x * by} to evaluate whether the
association between a continuous predictor and an outcome differs across
groups. Optionally produces a publication-ready visualization of
group-specific regression lines.
}
\details{
The interaction coefficient (\eqn{\beta}) represents the difference in
regression slopes between groups, conditional on the reference level
of \code{by}. The sign and magnitude of this coefficient depend on the
chosen reference group.

Confidence intervals are emphasized as the primary inferential quantity.
}
\examples{

# Simple example: different trends between groups

set.seed(123)

n <- 60

marker <- rnorm(n, 10, 2)
group  <- rep(c("Control", "Treatment"), each = n/2)

# Same intercept, different slopes
response <- 2 +
  ifelse(group == "Control", 0.5, 1.2) * marker +
  rnorm(n, 0, 1)

test.interaction(marker, response, group)

}
